






c                  Program CTW (Cylindrical Tank Walls)


c                             Version 1
c                               2013



c			      by A. Ghali
c                  Emeritus Professor of Civil Engineering

c			 The University of Calgary
c		       Calgary,Alberta,Canada,T2N1N4

c	Analysis of circular cylindrical walls using  finite differences.
c               Variable wall thickness. Circumferential line load.
c     Axi-symmetrical load distributed in arbitrary shape.  Thermal Effects.
c		     Prescribed edge displacements.


c                         Listing of program CTW

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION AK(101,3),w(101,8),EQ(101,8),AN(21,8),AM(21,8),Q(101,8)
      DIMENSION H(101)
      DIMENSION AKPRESC(101,3),wPRESC(101),EF(4)
      DIMENSION RN(2,8),RNS(2,8),F(101,8),AMULT(8),AMPHI(21,8)
      DIMENSION LTYPE(8),TO(8),TI(8)
      DIMENSION IR(21)
      CHARACTER  title*76,filnam*30

      INTEGER*2 ida,imo,iy

 5    print 6
 6    format(' Input  file name : ',\)
      read (*,'(A)') filnam
      OPEN(5,FORM='FORMATTED',MODE='READ',FILE= filnam,ERR=5)
 7    print 8
 8    format(' Output file name : ',\)
      read (*,'(A)') filnam
      OPEN(6,FORM='FORMATTED',MODE='WRITE',FILE= filnam,ERR=7)
      CALL GETDAT (IY,IMO,IDA)
      WRITE (6,9) IMO,IDA,IY
 9    format (i4.2,1h:,i2.2,1h:,i4,20x,'CTW Version 1.0'/)

      LPRESC=0
      WB=0.0
      THETAB=0.0
      WT=0.0
      THETAT=0.0
      DO 151 I=1,21
151   IR(I)=0

      DO 15 I=1,101
      DO 10 J=1,3
      AK(I,J)=0.0
10    AKPRESC(I,J)=0.0
      DO 15 J=1,8
      w(I,J)=0.
      Q(I,J)=0.
15    EQ(I,J)=0.
      DO 11 I=1,101
11    wPRESC(I)=0.0
      SSB=0.
      SST=0.
      DO 20 I=1,21
      DO 20 J=1,8
      AM(I,J)=0.
20    AN(I,J)=0.
      DO 19 I=1,8
      TO(I)=0.
19    TI(I)=0.
      AMBOT=0.
      AMTOP=0.
      RBOT=0.
      RTOP=0.
      DO 21 I=1,8
21    LTYPE(I)=99

      READ(5,100)TITLE
100   FORMAT(A76)
      WRITE(6,200)TITLE
200   FORMAT(3X,A76)

      READ(5,*)E,PR,ALPHA
      WRITE(6,300)E,PR,ALPHA
300   FORMAT(3X,'Elasticity modulus=',E11.5,1X,'; Poissons ratio=',F5.4,
     +/,3X,'Thermal expansion coefficient=',E11.5)

      READ(5,*)R,AL
      WRITE(6,400)R,AL
400   FORMAT(3X,'Tank raduis=',E11.5,1X,
     *'; height=', E11.5)
      ALAMDA=AL/100.

c     Read a set of lines describing thickness variation. Each line gives
c     three x-values followed  by three  thickness  values.  The  program
c     will find by  interpolation  values of thickness at nodes  situated
c     between  the  first  and	last x-values.	The interpolation assumes
c     parabolic variation   of thickness. Three values of x and thickness
c     must  be entered	even when  the	thickness varies linearly. Repeat
c     this  line  of  data as many times  as necessary to cover the whole
c     wall  height. Thus, the  first x-value ( on first line of this set)
c     must be  zero, and the  last  x-value (on last line of set) must be
c     equal to wall height, AL. The first x-value on any line must be the
c     smallest of the three x-values.
      WRITE(6,350)
350   FORMAT(/,3X,'Thickness variation',/,
     +	       3X,'    X1           X2          X3          h1',
     +	       '          h2          h3')

      XSUDDEN=0.
      XSTART=0.
      HEND=0.
      ICOUNT=0
12    CONTINUE
      READ(5,*)X1,X2,X3,H1,H2,H3
      IF(X1.NE.XSTART)WRITE(6,121)
121   FORMAT(/,3X,
     +'ERROR:The value of X1 must be equal to zero on the first line ',
     +/,3X,'defining the wall thickness variation .On other lines,the',/
     +/,3X,'value of X1 must  equal X3 given on the preceeding line.')
      IF(X1.NE.XSTART)STOP 99
      IF(HEND.EQ.0.0)GO TO 122
      IF(HEND.EQ.H1 )GO TO 122
c     Values of H given indicate a sudden change in thickness at X1.
c     Call this value XSUDDEN.
      ICOUNT=ICOUNT+1
      IF(ICOUNT.GT.1)WRITE(6,123)
123   FORMAT(/,3X,
     +'ERROR: The program allows no more than one sudden change of ',
     +/,3X,'wall thickness.')
      IF(ICOUNT.GT.1)STOP 99
      XSUDDEN=X1
C     Calculate the difference in the square of the thicknesses at the
c     sudden change, to be used in subroutine TEMPERAT.
      HSQDIF=HEND**2-H1**2
122   CONTINUE
      WRITE(6,351)X1,X2,X3,H1,H2,H3
351   FORMAT(3X,6(E10.5,2X))
      CALL LAGRANH(X1,X2,X3,H1,H2,H3,ALAMDA,H)
      XSTART=X3
      HEND=H3
      IF(X3.LT.AL)GO TO 12

      WRITE(6,3)
3     FORMAT(/,3X,'Thickness of wall at 101 equally-spaced nodes',/)
      DO 2 I=1,20
      J=(I-1)*5+1
      WRITE(6,4)J,H(J),J+1,H(J+1),J+2,H(J+2),J+3,H(J+3),J+4,H(J+4)
4     FORMAT(3X,5(I3,1X,F7.3,3X) )
2     CONTINUE
      WRITE(6,4)101,H(101)

      BETA= (3.*(1.-PR*PR))**0.25/SQRT(R*H(101))

      WRITE(6,4001)BETA,H(101)
4001  FORMAT(/,3X,'Beta = ',E11.5,
     * ', based on thickness at bottom (',F7.3,')'   )

      READ(5,*)NLC,IRESULTS
      WRITE(6,50)NLC,IRESULTS
c     NLC must not exceed 8
50    FORMAT(/,3X,'Number of load cases=',I2,'  ;IRESULTS=',I1,//,3X,
     +'The results are given at 21 nodes. When IRESULTS=0, the nodes ',
     +/,3X,
     +'are equally spaced. When IRESULTS=1,the 21 nodes can be selected'
     +,/,3X,'by the program user.',/)
      READ(5,*)IBB,SSB
      READ(5,*)IBT,SST
      WRITE(6,51)IBB,SSB
      WRITE(6,52)IBT,SST
51    FORMAT(3X,'Bottom boundary condition:IBB=',I1 ,'; spring stiffness
     +=',E11.5)
52    FORMAT(3X,'Top boundary condition:IBT=',I1,' ;  spring stiffness
     +=',E11.5)
      WRITE(6,700)
700   FORMAT(/,3X,
     +'Boundary condition indicators IBB and IBT are either 0 or 1;',/,
     +3X,'0 means that the end rotation is prevented;',/,
     +3X,'1 means that the end rotation is free.',/,
     +3X,'A spring support is assumed in all cases.',/,
     +3X,'The above boundary conditions do not apply to a case of prescr
     +ibed edge',/,3X,'displacement.')

c     To represent a rigid support ,enter a large value for SSB or SST
c     (e.g. 1.e9).A zero value for any of the same variables,indicates
c     that the transverse deflection is free.

c     Generate equivalent stiffness matrix.[AK]

      CALL STIFF(E,PR,R,ALAMDA,H,AK,XSUDDEN)

      DO 16 I=1,101
      DO 16 J=1,3
16    AKPRESC(I,J)=AK(I,J)

      CALL BOUNDARY(AK,E,PR,R,ALAMDA,H,IBB,SSB,IBT,SST)


c     Load data

c     Five load types are accepted:

c     LTYPE(I)=0 indicates a circumferential line load, whose location
c     is defined by x=VAR1 and intensity Q=VAR2 (force/length).

c     LTYPE(I)= -1 to  -9  indictes a distributed  load  with  intensity
c     q=VAR1 multiplied by one ofthe shape functions -1 to -9, defined in
c     subroutine SHAPES. The value VAR2 is dummy for this type.

c     LTYPE(I)=1 indicates a distributed load whose intensity is specified
c     at an odd number of points, not less than 3. The value VAR1 is x at
c     one point and VAR2 is its load intensity.The data lines specifying
c     the distributed load for a case of loading must follow consecutively.
c     the x-values must be listed in ascending order.

c     LTYPE(I)=2 indicates temperature rise varying linearly between VAR1
c     at outer face to VAR2 at inner face.

c     LTYPE(I)=3 indicates displacements prescribed at bottom edqe. VAR1
c     and VAR2 indicate respectively the radial outward  translation , w
c     and the rotation theta (= dw/dx); where x is the distance from the
c     the bottom edge to  any  point .

c     LTYPE(I)=4 indicates displacements prescribed  at  top  edqe . The
c     parameters VAR1 and VAR2 have the same meaning as  for LTYPE(I)=3.

c     With the sign convention adopted here a positive theta prescribed
c     at the bottom edge  will	produce an outward  deflection	in  the
c     immediate vicinity of the edge . But a positive theta  prescribed
c     at the top edge will cause the wall to deflect  inwards near  the
c     edge .

c     The values of VAR1 and VAR2 , representing the magnitudes of  the
c     prescribed displacements can be any real numbers other than  0.0 .
c     The  value 0.0 is reserved to indicate a free  displacement . The
c     following are two example data lines and their meaning :
c	 8    3    1.0	  1.0e-6
c	 8    4    0.0	  0.0
c     The two lines indicate that for case of loading 8, a unit owtward
c     radial  translation  is  prescribed  at the bottom edge while the
c     rotation at the same edge is restricted to the  negligible  value
c     1.0e-6 . In the same load  case , the  top  edge	tranlation  and
c     translation and rotation are free to occur. Absence of the second
c     data line has no effect ; the free  displacement	condition  will
c     be assumed by the program.

c     Note that only one case of prescribed displacements,at one or both
c     edges , can be analyzed in one run . The prescribed  displacements
c     apply only for the case considered . For other load cases analyzed
c     in the same run,	the edge conditions are as specified earlier  by
c     IBB, SSB, IBT and SST . No other external forces or  temperature
c     loading can be introduced in the case of prescribed edge	displac-
c     ements.

c     The last data line of load data starts with an integer greater than
c     NLC followed by a dummy integer and two dummy real values.

      WRITE(6,450)
450   FORMAT(/,15X,'LOAD DATA',/,3X,
     +'LCASE',1X,'LTYPE',2X,'VARIABLE 1',2X,'VARIABLE 2')
25    CONTINUE
      READ(5,*)LCASE,LTYPE(LCASE),VAR1,VAR2
399   CONTINUE
      IF(LCASE.GT.NLC)WRITE(6,506)LCASE,LTYPE(LCASE),VAR1,VAR2
      IF(LCASE.GT.NLC)GO TO 29
      IF(LTYPE(LCASE).EQ.0)WRITE(6,500)LCASE,LTYPE(LCASE),VAR1,VAR2
      IF(LTYPE(LCASE).EQ.1)WRITE(6,501)LCASE,LTYPE(LCASE),VAR1,VAR2
      IF(LTYPE(LCASE).EQ.2)WRITE(6,502)LCASE,LTYPE(LCASE),VAR1,VAR2
      IF(LTYPE(LCASE).EQ.3)WRITE(6,503)LCASE,LTYPE(LCASE),VAR1,VAR2
      IF(LTYPE(LCASE).EQ.4)WRITE(6,504)LCASE,LTYPE(LCASE),VAR1,VAR2
      IF(LTYPE(LCASE).LT.0)WRITE(6,505)LCASE,LTYPE(LCASE),VAR1,VAR2
500   FORMAT(3X,I3,3X,I3,3X,2(E11.5,1X),
     +	    '; line load Q (force/length)')
501   FORMAT(3X,I3,3X,I3,3X,2(E11.5,1X),
     +	    ';dist. load q(force/length sq.)')
502   FORMAT(3X,I3,3X,I3,3X,2(E11.5,1X),
     +	    ';temperature rise To,Ti (degrees)')
503   FORMAT(3X,I3,3X,I3,3X,2(E11.5,1X),
     +	    ';bot displs. WB,THETAB(length and rad.)')
504   FORMAT(3X,I3,3X,I3,3X,2(E11.5,1X),
     +	    ';top displs. WT,THETAT(length and rad.)')
505   FORMAT(3X,I3,3X,I3,3X,2(E11.5,1X),
     +	    ';load shape function (force/length sq.)')
506   FORMAT(3X,I3,3X,I3,3X,2(E11.5,1X),
     +	    '.  Terminationon of load data.')


c     LCASE=load case number

29    CONTINUE

      IF(LCASE.GT.NLC)GO TO 40
c     Load type 3: prescribed  displacements at bottom edge
      IF(LTYPE(LCASE).EQ.3)GO TO 271
      IF(LTYPE(LCASE).NE.4)GO TO 272
271   CONTINUE
      IF(LPRESC.EQ.0)LPRESC=LCASE
      IF(LPRESC.EQ.LCASE)GO TO 14
      WRITE(6,13)
13    FORMAT(3X,/,'ERROR: Prescribed displacements can be entered only
     +in  one load case.')
      STOP 99

14    CONTINUE

      IF(LTYPE(LCASE).NE.3)GO TO 18
      WB=VAR1
      THETAB=VAR2

      GO TO 25

18    CONTINUE

c     Load type 4 : prescribed displacements at top edge

      WT=VAR1
      THETAT=VAR2

      GO TO 25
272   CONTINUE


c     Load type 2: temperature loading

      IF(LTYPE(LCASE).NE.2)GO TO 39
      TO(LCASE)=VAR1
      TI(LCASE)=VAR2
      CALL TEMPERAT(E,PR,ALPHA,H,R,ALAMDA,TO,TI,LCASE,IBB,IBT,
     + EQ,CB,CT,XSUDDEN,HSQDIF)
      GO TO 25
39    CONTINUE

c     Load type 0: a concentrated load

      IF(LTYPE(LCASE).NE.0) GO TO 31
      RN1=VAR1/ALAMDA
      N1=INT(RN1)+1


      EQ(N1,LCASE)=EQ(N1,LCASE)+VAR2*(N1*ALAMDA-VAR1)/ALAMDA
      EQ(N1+1,LCASE)=EQ(N1+1,LCASE)+VAR2*(VAR1-(N1-1)*ALAMDA)/ALAMDA


      GO TO 25
31    CONTINUE

c     LTYPE 1 : a distributed load with intensity specified by data lines

      IF(LTYPE(LCASE).NE.1)GO TO 32
      LC=LCASE
      X1=VAR1
      Y1=VAR2
      READ(5,*)LCASE,LTYPE(LCASE),VAR1,VAR2
      WRITE(6,501)LCASE,LTYPE(LCASE),VAR1,VAR2
      IF(LCASE.NE.LC)WRITE(6,97)
97    FORMAT(/,3X,
     +'ERROR:Distributed load intensity, q must be specified at an odd',
     +/,3X,'number of points. Minimum number of points=3.')
      IF(LCASE.NE.LC)STOP 99
      IF(LTYPE(LCASE).NE.1)WRITE(6,97)
      IF(LTYPE(LCASE).NE.1)STOP 99
35    CONTINUE
      X2=VAR1
      Y2=VAR2
      READ(5,*)LCASE,LTYPE(LCASE),VAR1,VAR2
      WRITE(6,501)LCASE,LTYPE(LCASE) ,VAR1,VAR2
      IF(LCASE.NE.LC)WRITE(6,97)
      IF(LCASE.NE.LC)STOP 99
      IF(LTYPE(LCASE).NE.1)WRITE(6,97)
      IF(LTYPE(LCASE).NE.1)STOP 99
      X3=VAR1
      Y3=VAR2
      CALL LAGRANGE(X1,X2,X3,Y1,Y2,Y3,ALAMDA,Q,LCASE)
      X1=X3
      Y1=Y3
      READ(5,*)LCASE,LTYPE(LCASE),VAR1,VAR2
      IF(LCASE.NE.LC)GO TO 399
      IF(LTYPE(LCASE).EQ.1)
     + WRITE(6,501)LCASE,LTYPE(LCASE) ,VAR1,VAR2
      GO TO 35
32    CONTINUE

c     LTYPE -1 to -8:load intensity varying according to shape functions

      IF(LTYPE(LCASE).GT.4)WRITE(6,322)LTYPE(LCASE)
      IF(LTYPE(LCASE).GT.4)STOP 99
      IF(LTYPE(LCASE).LT.-9)WRITE(6,322)LTYPE(LCASE)
322   FORMAT(/,3X,'ERROR : Load type',I4,' is not recognized.')
      IF(LTYPE(LCASE).LT.-9)STOP 99
321   CONTINUE

      CALL SHAPES(LCASE,LTYPE ,VAR1,Q,ALAMDA,AL,BETA)
      GO TO 25
40    CONTINUE

      IF(LPRESC.EQ.0)GO TO 401
      WRITE(6,418)LPRESC
418   FORMAT(/,3X,'BOUNDARY CONDITION INDICATORS FOR LCASE',I2,' ONLY',
     +/,/,3X,'Load case',5X,'Edge',6X,'ITRAN', 3X,'IROTN')
408   CONTINUE
      ITRAN=0
      IROTN=0
      IF(WB.EQ.0.0)ITRAN=1
      IF(THETAB.EQ.0.0)IROTN=1
      WRITE(6,419)LPRESC,ITRAN,IROTN
419   FORMAT(7X,I2,7X,'Bottom',6X,I2,6X,I2)
      ITRAN=0
      IROTN=0
      IF(WT.EQ.0.0)ITRAN=1
      IF(THETAT.EQ.0.0)IROTN=1
      WRITE(6,403)LPRESC,ITRAN,IROTN
403   FORMAT(7X,I2,7X,' Top  ',6X,I2,6X,I2)


      WRITE (6,412)
412   FORMAT(
     + /,3X,'ITRAN= 0 means the edge translation = VARIABLE 1.',
     +/,3X,'IROTN= 0 means the edge rotation = VARIABLE 2.',
     +/,3X,'ITRAN or IROTN= 1 indicates free displacement.')

      CALL PRESC(E,PR,R,ALAMDA,H,WB,THETAB,WT,THETAT,wPRESC,EF)

401   CONTINUE

      DO 27 J=1,NLC

      WRITE(6,88)J
88    FORMAT(/,3X,'LOAD CASE NO. ',I1,/,3X,
     +'External applied load intensity at 21 equally spaced points.',
     +/,3X,'Units are force/length sq.:',/)
27    WRITE(6,77)(Q(I,J),I=1,101,5)

77    FORMAT(3X,5E13.5)

c     Equivalent concentrated loads to replace distributed loads
      CALL EQUIVALENT(Q,NLC,EQ,ALAMDA)

      DO 28 J=1,NLC
      WRITE(6,89)J
89    FORMAT(/,3X,' LOAD CASE NO. ',I1)
c     IF(LTYPE(J).EQ.0)WRITE(6,890)
c     IF(LTYPE(J).EQ.1)WRITE(6,891)
c     IF(LTYPE(J).EQ.2)WRITE(6,892)
c     IF(LTYPE(J).EQ.3)WRITE(6,893)
c     IF(LTYPE(J).EQ.4)WRITE(6,894)
c     IF(LTYPE(J).LT.0)WRITE(6,895)
890   FORMAT(20X,'CONCENTRATED LINE LOAD',/)
891   FORMAT(20X,'DISTRIBUTED LOAD',/)
892   FORMAT(20X,'THERMAL LOAD',/)
893   FORMAT(20X,'PRESCRIBED EDGE DISPLACEMENTS '/)
894   FORMAT(20X,'PRESCRIBED EDGE DISPLACEMENTS '/)
895   FORMAT(20X,'LOAD DISTRIBUTED ACCORDING TO A SHAPE FUNCTION'/)

      WRITE(6,896)
896   FORMAT(3X,
     +'Equivalent concentrated forces at 101 equally-spaced nodes',/,
     +3X,'Units are force/length:',/)
      DO 793 I=1,25
      k=(I-1)*4+1
      WRITE(6,794)k,EQ(k,J),
     +		k+1,EQ(k+1,J),k+2,EQ(k+2,J),k+3,EQ(k+3,J)
794   FORMAT(3X,4(I3,1X,E11.5,3X) )
793   CONTINUE
      WRITE(6,794)101,EQ(101,J)


28    CONTINUE

c     Solution of equilibrium equations
      NDF=101
      NB=3
      DO 83 I=1,101
      DO 83 J=1,8
83    F(I,J)=-EQ(I,J)

      CALL EQSOLN(AK,F,w,NDF,NB,NLC)

c     When  edge displacements	are prescribed, replace the appropriate
c     column of the matrx w by the values calculated in the  subroutine
c     PRESC .

      IF(LPRESC.EQ.0)GO TO 84
      DO 86 I=1,101
86    w(I,LPRESC)=wPRESC(I)
84    CONTINUE

c     Calculate and print hoop forces,bending moments in vertical
c     and circumpherential directions and end reactions.

      IF(IRESULTS.EQ.1)READ(5,*)(IR(K),K=1,21)

      CALL INTFORCES(w,E,R,H,PR,ALAMDA,AN,AM,RN,IBB,IBT,EQ,NLC,
     +SSB,SST,AMPHI,LTYPE,RNS,LPRESC,EF,TO,TI,ALPHA,XSUDDEN,IR,IRESULTS)


c     Combine the NLC load cases multiplied by given values.

903   CONTINUE
      READ(5,*)LCOMBN,(AMULT(J),J=1,NLC)
      IF(LCOMBN.EQ.99)GO TO 902

      CALL COMBINE(LCOMBN,AN,AM,RN,AMPHI,AMULT,ALAMDA,NLC,Q,EQ,
     +	  IR,IRESULTS)

      GO TO 903
902   CONTINUE
99    CONTINUE
      CLOSE(5)
      CLOSE(6)
      STOP
      END


      SUBROUTINE STIFF(E,PR,R,ALAMDA,H,AK,XSUDDEN)

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c     Generate equivalent stiffness matrix by Eq.3.8 of Ghali,Circular
c     Storage tanks and Silos,Spon,1979. Both ends free.

      DIMENSION AK(101,3) , H(101) ,AJ(101) ,C(101)
      EIB=E*(H(1)**3)/(12.*(1-PR*PR))
      DO 20 I=1,101
      AJ(I)=(H(I)/H(1))**3
      AKI=E*H(I)/(R*R)
20    C(I)=AKI*(ALAMDA**4)/EIB

      IF(XSUDDEN.EQ.0.)GO TO 11
C     Find out the number of the node closest to the sudden thickness
C     change. Calculate an effctive relative second moment of area(see
C     Ghali and Neville, Structural Analysis, 3rd ed.,1989,p.495.)
      ISUDDEN=IDINT(XSUDDEN/ALAMDA)+1
      REMAIND=XSUDDEN-ALAMDA*ISUDDEN
      AKI=E*(H(ISUDDEN)+H(ISUDDEN+1))/(2.*R*R)
      IF(REMAIND.GE.(1.-REMAIND))
     + C(ISUDDEN+1)=AKI*(ALAMDA**4)/EIB
      IF(REMAIND.LT.(1.-REMAIND))
     + C(ISUDDEN)=AKI*(ALAMDA**4)/EIB

      IF(REMAIND.GE.(1.-REMAIND))
     + AJ(ISUDDEN+1)=2.*AJ(ISUDDEN)*AJ(ISUDDEN+1)/
     +		       ( AJ(ISUDDEN)+AJ(ISUDDEN+1 ))

      IF(REMAIND.LT.(1.-REMAIND))
     + AJ(ISUDDEN)=2.*AJ(ISUDDEN)*AJ(ISUDDEN+1)/
     +		       ( AJ(ISUDDEN)+AJ(ISUDDEN+1 ))
11    CONTINUE

      CONST=EIB/(ALAMDA**3)

      AK(1,1)=CONST*(AJ(2)+C(1)/2.)
      AK(101,1)=CONST*(AJ(100)+C(101)/2.)
      AK(1,2)=-2.*CONST*AJ(2)
      AK(1,3)=CONST*AJ(2)
      AK(2,1)=CONST*(4.*AJ(2)+AJ(3)+C(2))
      AK(2,2)=CONST*(-2.*AJ(2)-2.*AJ(3))
      AK(2,3)=CONST*AJ(3)
      AK(100,1)=CONST*(AJ(99)+4.*AJ(100)+C(100))
      AK(100,2)=-2.*CONST*AJ(100)
      DO 10 I=3,99
      AK(I,1)=CONST*(AJ(I-1)+4.*AJ(I)+AJ(I+1)+C(I))
      AK(I,2)=-2.*CONST*(AJ(I)+AJ(I+1))
10    AK(I,3)=CONST*AJ(I+1)
      RETURN
      END

      SUBROUTINE BOUNDARY(AK,E,PR,R,ALAMDA,H,IBB,SSB,IBT,SST)
c     Adjust [AK] for boundary conditions at top and bottom
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION AK(101,3)
      DIMENSION  H(101) ,AJ(101) ,C(101)
      EIB=E*(H(1)**3)/(12.*(1-PR*PR))
      DO 20 I=1,3
      AJ(I)=(H(I)/H(1))**3
      AKI=E*H(I)/(R*R)
20    C(I)=AKI*(ALAMDA**4)/EIB
      DO 30 I=99,101
      AJ(I)=(H(I)/H(1))**3
      AKI=E*H(I)/(R*R)
30    C(I)=AKI*(ALAMDA**4)/EIB

      CONST=EIB/(ALAMDA**3)
      AK(1,1)=AK(1,1)+SSB
      IF(IBB.EQ.0)AK(1,1)=CONST*(2.*AJ(1)+AJ(2)+C(1)/2.)+SSB
      IF(IBB.EQ.0)AK(1,2)=-2.*CONST*(AJ(1)+AJ(2))
      IF(IBB.EQ.0)AK(2,1)=CONST*(2.*AJ(1)+4.*AJ(2)+AJ(3)+C(2))

      AK(101,1)=AK(101,1)+SST
      IF(IBT.EQ.0)AK(101,1)=CONST*(2.*AJ(101)+AJ(100)+C(101)/2.)+SST
      IF(IBT.EQ.0)AK(100,1)=CONST*(2.*AJ(101)+4.*AJ(100)+AJ(99)+C(100))
      IF(IBT.EQ.0)AK(100,2)=-2.*CONST*(AJ(100)+AJ(101))

      RETURN
      END


      SUBROUTINE LAGRANGE(X1,X2,X3,Y1,Y2,Y3,ALAMDA,Q,LCASE)
c     Interpolation using Lagrange functions between three values of y.
c     Use the result to generate load intensity matrix,[Q].
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION Q(101,8)

      DO 10 I=1,101
      X=ALAMDA*(I-1)
      IF(X.LT.X1)GO TO 10
      IF(X.GT.X3*1.0000001)GO TO 10
      C1=((X-X2)/(X1-X2))*((X-X3)/(X1-X3))
      C2=((X-X1)/(X2-X1))*((X-X3)/(X2-X3))
      C3=((X-X1)/(X3-X1))*((X-X2)/(X3-X2))
      Q(I,LCASE)=Q(I,LCASE)+C1*Y1+C2*Y2+C3*Y3
10    CONTINUE
      RETURN
      END


      SUBROUTINE SHAPES(LCASE,LTYPE ,VAR1,Q,ALAMDA,AL,BETA)

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION Q(101,8),LTYPE(8)
      PI=3.141592654

      BLAMDA=7.203/AL

      DO 10 I=1,101
      X=ALAMDA*(I-1)
      XI=X/AL
      IF(LTYPE(LCASE).EQ.-1)S=SIN(PI*XI)
      IF(LTYPE(LCASE).EQ.-2)S=SIN(2*PI*XI)
      IF(LTYPE(LCASE).EQ.-3)S=EXP(-PI*XI)*SIN(PI*XI)
      IF(LTYPE(LCASE).EQ.-4)S=EXP(-2.*PI*XI)*SIN(2.*PI*XI)
      IF(LTYPE(LCASE).EQ.-5)S=EXP(-BETA*X)*COS(BETA*X)
      IF(LTYPE(LCASE).EQ.-6)S=EXP(-1.25*BETA*X)*COS(1.25*BETA*X)
      IF(LTYPE(LCASE).EQ.-7)S=EXP(-BETA*X)*SIN(BETA*X)
      IF(LTYPE(LCASE).EQ.-8)S=EXP(-1.25*BETA*X)*SIN(1.25*BETA*X)

c     The following shape function is suggested by Brondum-Nielson,ACI
c     Structural Journal,July-Aug.,1985.
      IF(LTYPE(LCASE).EQ.-9)S=AL-X-AL*EXP(-BLAMDA*X)*(COS(BLAMDA*X)+
     *	SIN(BLAMDA*X))+(1./BLAMDA)*EXP(-BLAMDA*X)*SIN(BLAMDA*X)

      Q(I,LCASE)=Q(I,LCASE)+S*VAR1
10    CONTINUE
      RETURN
      END

      SUBROUTINE EQUIVALENT(Q,NLC,EQ,ALAMDA)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION Q(101,8),EQ(101,8)
      DO 20 J=1,NLC
      EQ(1,J)=EQ(1,J)+ALAMDA*(7.*Q(1,J)+6.*Q(2,J)-Q(3,J))/24.
      DO 10 I=2,100
      EQ(I,J)=EQ(I,J)+ALAMDA*(Q(I-1,J)+10.*Q(I,J)+Q(I+1,J))/12.
10    CONTINUE
      EQ(101,J)=EQ(101,J)+ALAMDA*(7.*Q(101,J)+6.*Q(100,J)-Q(99,J))/24.
20    CONTINUE
      RETURN
      END

      SUBROUTINE INTFORCES(w,E,R,H,PR,ALAMDA,AN,AM,RN,IBB,IBT,EQ,NLC,
     +SSB,SST,AMPHI,LTYPE,RNS,LPRESC,EF,TO,TI,ALPHA,XSUDDEN,IR,IRESULTS)

c     Internal forces and reactions.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION w(101,8),AN(21,8),AM(21,8),RN(2,8),RNS(2,8),EQ(101,8)
      DIMENSION AMPHI(21,8)
      DIMENSION LTYPE(8),EF(4)
      DIMENSION H(101),C1(101),C2(101),AJ(101),C(101)
      DIMENSION TO(8),TI(8)
      DIMENSION IR(21)

      DO 13 I=1,101
      EI=E*(H(I)**3)/(12.*(1-PR*PR))
      C1(I)=EI/(ALAMDA*ALAMDA)
      C2(I)=E*H(I)/R
13    CONTINUE

      IF(XSUDDEN.EQ.0.)GO TO 112
C     Find out the number of the node closest to the sudden thickness
C     change. Calculate an effctive relative second moment of area(see
C     Ghali and Neville, Structural Analysis, 3rd ed.,1989,p.495.)
      ISUDDEN=IDINT(XSUDDEN/ALAMDA)+1
      REMAIND=XSUDDEN-ALAMDA*ISUDDEN
      IF(REMAIND.GE.(1.-REMAIND))
     + C1(ISUDDEN+1)=2.*C1(ISUDDEN)*C1(ISUDDEN+1)/
     +		       ( C1(ISUDDEN)+C1(ISUDDEN+1 ))

      IF(REMAIND.LT.(1.-REMAIND))
     + C1(ISUDDEN)=2.*C1(ISUDDEN)*C1(ISUDDEN+1)/
     +		       ( C1(ISUDDEN)+C1(ISUDDEN+1 ))
112    CONTINUE


      DO 99 J=1,NLC


c     Bending moments in vertical direction.
      IF(IRESULTS.EQ.0)GO TO 2
      IF(IR(1).NE.1)GO TO 20
2     CONTINUE
      IF(IBB.EQ.1) AM(1,J)=0.
10    IF(IBB.EQ.0)AM(1,J)=(-2.*w(1,J)+2.*w(2,J))*(-C1(1))
20    CONTINUE
      IF(IRESULTS.EQ.0)GO TO 3
      IF(IR(21).NE.101)GO TO 40
3     CONTINUE
      IF(IBT.EQ.1)
     +AM(21,J)=0.
 30   IF(IBT.EQ.0)AM(21,J)=(-2.*w(101,J)+2.*w(100,J))*(-C1(101))
 40   CONTINUE
      DO 50 I=2,20
      IF(IRESULTS.EQ.0)K=(I-1)*5+1
      IF(IRESULTS.EQ.1)K=IR(I)
 50   AM(I,J)=-C1(K)*(w(K-1,J)-2.*w(K,J)+w(K+1,J))
      IF(IRESULTS.EQ.0)GO TO 4
      IF(IR(1).NE.1)
     +AM(IR(1),J)=-C1(IR(1))*(w(IR(1)-1,J)-2.*w(IR(1),J)+w(IR(1)+1,J))
      IF(IR(21).NE.101)
     +AM(IR(21),J)=
     +	-C1(IR(21))*(w(IR(21)-1,J)-2.*w(IR(21),J)+w(IR(21)+1,J))
4     CONTINUE
c     In the case of  prescribed  edge	 displacements , the  bending
c     moments at the edges are calculated earlier by SUBROUTINE PRESC.

      IF(J.NE.LPRESC)GO TO 11
      IF(IRESULTS.EQ.0)GO TO 5
      IF(IR(1).NE.1)GO TO 101
5     CONTINUE
      AM(1,J)=EF(2)
101   CONTINUE
      IF(IRESULTS.EQ.0)GO TO 6
      IF(IR(21).NE.101)GO TO 11
6     CONTINUE
      AM(21,J)=-EF(4)
11    CONTINUE

c     Bending moments in circumpherential direction
      DO 51 I=1,21
51    AMPHI(I,J)=AM(I,J)*PR

c     Add bending moment in the circumpherential direction due to
c     temperature in the restraint condition.
      IF(LTYPE(J).NE.2)GO TO 46
      DO 45 I=1,21
      IF(IRESULTS.EQ.0)K=(I-1)*5+1
      IF(IRESULTS.EQ.1)K=IR(I)
      AMPHI(I,J)=AMPHI(I,J)-
     +		       (E*ALPHA*H(K)*H(K)/(1.-PR))*(TO(J)-TI(J))/12.
  45  CONTINUE

c     Elimination of an edge couple by two line loads ,as was done in
c     Subroutine TEMPERAT, does not give the  correct  moment  in the
c     circumpherential	direction  at  the edge .  The following is a
c     correction.
      IF(IRESULTS.EQ.0)GO TO 7
      IF(IR(1).NE.1)GO TO 102
7     CONTINUE
      IF(IBB.EQ.1)AMPHI(1,J)=AMPHI(1,J)+PR*
     +		     (E*ALPHA*H(1)*H(1)/(1.-PR))*(TO(J)-TI(J))/12.
102   CONTINUE
      IF(IRESULTS.EQ.0)GO TO 8
      IF(IR(21).NE.101)GO TO 46
8     CONTINUE
      IF(IBT.EQ.1)AMPHI(21,J)=AMPHI(21,J)+PR*
     +		     (E*ALPHA*H(101)*H(101)/(1.-PR))*(TO(J)-TI(J))/12.
  46  CONTINUE


c     Reactions at ends
      EIB=E*H(1)**3/(12.*(1.-PR*PR))
      DO 47 I=1,101
      AJ(I)=(H(I)/H(1) )**3
47    C(I)=(E*H(I)/(R*R))*(ALAMDA**4)/EIB

      IF(XSUDDEN.EQ.0.)GO TO 114
C     Find out the number of the node closest to the sudden thickness
C     change. Calculate an effctive relative second moment of area(see
C     Ghali and Neville, Structural Analysis, 3rd ed.,1989,p.495.)
      ISUDDEN=IDINT(XSUDDEN/ALAMDA)+1
      REMAIND=XSUDDEN-ALAMDA*ISUDDEN

      IF(REMAIND.GE.(1.-REMAIND))
     + AJ(ISUDDEN+1)=2.*AJ(ISUDDEN)*AJ(ISUDDEN+1)/
     +		       ( AJ(ISUDDEN)+AJ(ISUDDEN+1 ))
      IF(REMAIND.GE.(1.-REMAIND))
     + C(ISUDDEN+1)=C(ISUDDEN)+C(ISUDDEN+1)/2.

      IF(REMAIND.LT.(1.-REMAIND))
     + AJ(ISUDDEN)=2.*AJ(ISUDDEN)*AJ(ISUDDEN+1)/
     +		       ( AJ(ISUDDEN)+AJ(ISUDDEN+1 ))
      IF(REMAIND.LT.(1.-REMAIND))
     + C(ISUDDEN)=C(ISUDDEN)+C(ISUDDEN+1)/2.

114   CONTINUE

      IF(IBB.EQ.1)
     +RN(1,J)=EQ(1,J)+( EIB/(ALAMDA**3) )*
     +	(  -(AJ(2)+C(1)/2.)*w(1,J)+2.*AJ(2)*w(2,J)-AJ(2)*w(3,J) )
60    IF(IBB.EQ.0)
     +RN(1,J)=EQ(1,J)+( EIB/(ALAMDA**3) )*
     +	(  -(2.*AJ(1)+AJ(2)+C(1)/2.)*w(1,J)+2.*(AJ(1)+AJ(2))*w(2,J)
     +					   -AJ(2)*w(3,J)  )

70    CONTINUE
      IF(IBT.EQ.1)
     +RN(2,J)=EQ(101,J)+( EIB/(ALAMDA**3) )*
     +	(  -(AJ(100)+C(101)/2.)*w(101,J)+2.*AJ(100)*w(100,J)
     +					 -AJ(100)*w(99,J) )
80    IF(IBT.EQ.0)
     +RN(2,J)=EQ(101,J)+( EIB/(ALAMDA**3) )*
     +	(  -(2.*AJ(101)+AJ(100)+C(101)/2.)*w(101,J)+
     +		2.*(AJ(101)+AJ(100))*w(100,J)- AJ(100)*w(99,J)	)
90    CONTINUE


c     Hoop forces
      DO 95 I=1,21
      IF(IRESULTS.EQ.0)K=(I-1)*5+1
      IF(IRESULTS.EQ.1)K=IR(I)
95    AN(I,J)=C2(K)*w(K,J)

c     Add hoop force due to temperature in the restraint condition.
      IF(LTYPE(J).NE.2)GO TO 66
      DO 65 I=1,21
      IF(IRESULTS.EQ.0)K=(I-1)*5+1
      IF(IRESULTS.EQ.1)K=IR(I)
      AN(I,J)=AN(I,J)-(E*ALPHA*H(K)*(TO(J)+TI(J))/2.)
  65  CONTINUE

  66  CONTINUE

c     Add bending moment in vertical direction due to temperature in the
c      restraint condition

      IF(LTYPE(J).NE.2)GO TO 56
      DO 55 I=1,21
      IF(IRESULTS.EQ.0)K=(I-1)*5+1
      IF(IRESULTS.EQ.1)K=IR(I)
      AM(I,J)=AM(I,J)-(E*ALPHA*H(K)*H(K)/(1.-PR))*(TO(J)-TI(J))/12.
  55  CONTINUE

c     Elimination of an edge couple by two line loads ,as was done in
c     Subroutine TEMPERAT, does not give the  correct  moment in  the
c     vertical direction  at the edge. The following is a correction.
      IF(IRESULTS.EQ.0)GO TO 9
      IF(IR(1).NE.1)GO TO 103
9     CONTINUE
      IF(IBB.EQ.1)AM(1,J)=AM(1,J)+
     +		     (E*ALPHA*H(1)*H(1)/(1.-PR))*(TO(J)-TI(J))/12.
103   CONTINUE
      IF(IRESULTS.EQ.0)GO TO 19
      IF(IR(21).NE.101)GO TO 56
19    CONTINUE
      IF(IBT.EQ.1)AM(21,J)=AM(21,J)+
     +		     (E*ALPHA*H(101)*H(101)/(1.-PR))*(TO(J)-TI(J))/12.
  56  CONTINUE


99    CONTINUE
      DO 109 J=1,NLC
      WRITE(6,100)J
100   FORMAT(/,/,25X,'LOAD CASE',I2)

      IF(LTYPE(J).EQ.3)WRITE(6,893)
      IF(LTYPE(J).EQ.4)WRITE(6,894)

890   FORMAT(20X,'CONCENTRATED LINE LOAD',/)
891   FORMAT(20X,'DISTRIBUTED LOAD',/)
892   FORMAT(20X,'THERMAL LOAD',/)
893   FORMAT(25X,'PRESCRIBED EDGE DISPLACEMENTS '/)
894   FORMAT(25X,'PRESCRIBED EDGE DISPLACEMENTS '/)
895   FORMAT(20X,'LOAD DISTRIBUTED ACCORDING TO A SHAPE FUNCTION'/)

      WRITE(6,200)
200   FORMAT(/,3X,'Node',1X,'Distance fr. bot.',1X, 'Hoop force',1X,
     +'Bending moment,Mx',1X,'Bending moment,Mphi')
      DO 107 I=1,21
      IF(IRESULTS.EQ.0)X=(5*(I-1))*ALAMDA
      IF(IRESULTS.EQ.0)NODE=5*(I-1)+1
      IF(IRESULTS.EQ.1)X=(IR(I)-1)*ALAMDA
      IF(IRESULTS.EQ.1)NODE=IR(I)
107   WRITE(6,300)NODE,X,AN(I,J),AM(I,J),AMPHI(I,J)
300   FORMAT(3X,I3,4X,F7.3,6X,E13.5,3X,E13.5,5X,E13.5)
      WRITE(6,400)
400   FORMAT(/,3X,'Reactions:')

c     In the case of prescribed  displacements	at edges , the reactions
c     at the edges are calculated earlier by SUBROUTINE PRESC.

      IF(J.NE.LPRESC)GO TO 111
      RN(1,J)=-EF(1)

      RN(2,J)=-EF(3)
      WRITE(6,500)RN(1,J),RN(2,J)
500   FORMAT(/,3X,'Reaction at bottom=',E13.5,
     + '; reaction at top=',E13.5)
      GO TO 109
111   CONTINUE

c     WRITE(6,401)
401   FORMAT(/,3X,'Reactions calculated using spring stiffnesses:')
      RNS(1,J)=w(1,J)*SSB
      RNS(2,J)=w(101,J)*SST
      WRITE(6,500)RNS(1,J),RNS(2,J)

109   CONTINUE

      RETURN
      END

      SUBROUTINE COMBINE(LCOMBN,AN,AM,RN,AMPHI,AMULT,ALAMDA,NLC,Q,EQ,
     +	IR,IRESULTS)
c     Combine cases 1 to NLC multiplying by load factors
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION AN(21,8),AM(21,8),RN(2,8),AMPHI(21,8),AMULT(8)
      DIMENSION Q(101,8),EQ(101,8)
      DIMENSION IR(21)
      WRITE(6,100)LCOMBN
100   FORMAT(/,28X,'LOAD COMBINATION NO.',I2)

      WRITE(6,901) (AMULT(J),J=1,NLC)
901   FORMAT(/,3X,'Load factors:',/,/,3X,
     +4E13.5,/,3X,4E13.5)

      WRITE(6,250)
250   FORMAT(/,3X,'NODE Distance from bot.',4X, 'Load intensity',5X,
     +'Equivalent concent. force')
      DO 25 I=1,101
      QC=0.
      EQC=0.
      X=(I-1)*ALAMDA
      DO 35 J=1,NLC
      QC=QC+Q(I,J)*AMULT(J)
35    EQC=EQC+EQ(I,J)*AMULT(J)
      WRITE(6,350)I,X,QC,EQC
350   FORMAT(3X,I3,5X,F7.3,10X,E13.5,10X,E13.5)
25    CONTINUE
      WRITE(6,200)
      DO 20 I=1,21
      ANC=0.
      AMC=0.
      AMPHIC=0.
      IF(IRESULTS.EQ.0)X=(5*(I-1))*ALAMDA
      IF(IRESULTS.EQ.0)NODE=(I-1)*5+1
      IF(IRESULTS.EQ.1)X=(IR(I)-1)*ALAMDA
      IF(IRESULTS.EQ.1)NODE=IR(I)
      DO 10 J=1,NLC
      ANC=ANC+AN(I,J)*AMULT(J)
      AMC=AMC+AM(I,J)*AMULT(J)
10    AMPHIC=AMPHIC+AMPHI(I,J)*AMULT(J)
      WRITE(6,300)NODE,X,ANC,AMC,AMPHIC
20    CONTINUE
200   FORMAT(/,3X,'Node',1X,'Distance fr. bot.',1X, 'Hoop force',1X,
     +'Bending moment,Mx',1X,'Bending moment,Mphi')

300   FORMAT(3X,I3,4X,F7.3,6X,E13.5,3X,E13.5,5X,E13.5)
      RBC=0.
      RTC=0.
      DO 30 J=1,NLC
      RBC=RBC+RN(1,J)*AMULT(J)
30    RTC=RTC+RN(2,J)*AMULT(J)
      WRITE(6,400)
400   FORMAT(/,3X,'Reactions:')
      WRITE(6,500)RBC,RTC
500   FORMAT(/,3X,'Reaction at bottom=',E13.5,'; reaction at top=',
     + E13.5)

      RETURN
      END


      SUBROUTINE TEMPERAT(E,PR,ALPHA,H,R,ALAMDA,TO,TI,LCASE,IBB,IBT,
     +	EQ,CB,CT,XSUDDEN,HSQDIF)


      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION EQ(101,8)
      DIMENSION H(101),RP(101),TO(8),TI(8)

      DO 5 I=1,101
5     RP(I)=0.0
c     Internal forces and restraining pressure when radial expansion and edge
c     rotations are artificially prevented.

      DO 15 I=2,100
      ANRI=-(E*ALPHA*H(I))*((TO(LCASE)+TI(LCASE))/2.)
      AMRI=-(E*ALPHA*H(I)*H(I)/(1.-PR))*((TO(LCASE)-TI(LCASE))/12.)
      AMRIM=-(E*ALPHA*H(I-1)*H(I-1)/(1.-PR))*((TO(LCASE)-TI(LCASE))/12.)
      AMRIP=-(E*ALPHA*H(I+1)*H(I+1)/(1.-PR))*((TO(LCASE)-TI(LCASE))/12.)
      RP(I)=ANRI/R-(AMRIM-2.*AMRI+ AMRIP )/(ALAMDA**2)
15    CONTINUE
c     Restraining pressure  at first and last nodes.
      ANR1=-(E*ALPHA*H(1))*((TO(LCASE)+TI(LCASE))/2.)
      AMR1=-(E*ALPHA*H(1)*H(1)/(1.-PR))*((TO(LCASE)-TI(LCASE))/12.)
      AMR2=-(E*ALPHA*H(2)*H(2)/(1.-PR))*((TO(LCASE)-TI(LCASE))/12.)
      AMR3=-(E*ALPHA*H(3)*H(3)/(1.-PR))*((TO(LCASE)-TI(LCASE))/12.)
      RP(1)=ANR1/R-(AMR1-2.*AMR2+AMR3)/(ALAMDA**2)

      ANR101=-(E*ALPHA*H(101))*((TO(LCASE)+TI(LCASE))/2.)
      AMR101=-(E*ALPHA*H(101)*H(101)/(1.-PR))*
     +			 ((TO(LCASE)-TI(LCASE))/12.)
      AMR100=-(E*ALPHA*H(100)*H(100)/(1.-PR))*
     +			 ((TO(LCASE)-TI(LCASE))/12.)
      AMR99 =-(E*ALPHA*H(99 )*H(99 )/(1.-PR))*
     +			 ((TO(LCASE)-TI(LCASE))/12.)
      RP(101)=ANR101/R-(AMR101-2.*AMR100+AMR99 )/(ALAMDA**2)

      IF(XSUDDEN.EQ.0.)GO TO 11

C     Find out the number of the nodes ISUDDEN and ISUDDEN+1 on either side
C     of the sudden thickness change. As approximation consider the second
C     derivative of (h sq.) at each of the two nodes to be the same as at
C     the adjacent node on either side.

      ISUDDEN=IDINT(XSUDDEN/ALAMDA)+1
      RP(ISUDDEN)=RP(ISUDDEN-1)
      RP(ISUDDEN+1)=RP(ISUDDEN+2)
11    CONTINUE

c     Restraining couples at bottom and top edges

      CB=AMR1
      CT=-AMR101

c     Restraining forces at edges(see Eq. 3.33 of Ghali, A., Circular
c     Storage Tanks and Silos,Spon,London,1979,pp. 210.

      F1r=(ALPHA*E*(TO(LCASE)-TI(LCASE))/(12.*(1-PR)))*
     +	    ( (-3.*H(1)*H(1)+4.*H(2)*H(2)-H(3)*H(3))/(2.*ALAMDA) )


      F101r=-(ALPHA*E*(TO(LCASE)-TI(LCASE))/(12.*(1-PR)))*
     + ((H(99)*H(99)-4.*H(100)*H(100)+3.*H(101)*H(101))/(2.*ALAMDA) )
c     Eliminate artificial pressure and edge forces

      EQ(1,LCASE)=EQ(1,LCASE)-
     +	       ALAMDA*(7.*RP(1)+6.*RP(2)-RP(3))/24.
      EQ(101,LCASE)=EQ(101,LCASE)-
     +	   ALAMDA*(7.*RP(101)+6.*RP(100)-RP(99))/24.
      DO 10 I=2,100
c     Equivalent nodal forces at the two nodes on either side of
c     a sudden thichness change.
      IF(XSUDDEN.EQ.0.)GO TO 171
      ISUDDEN=IDINT(XSUDDEN/ALAMDA)+1
      REMAIND=XSUDDEN-ISUDDEN*ALAMDA
c     Determine which of the nodes ISUDDEN or ISUDDEN+1 is closer to
c     the sudden thickness change.
      IF(I.NE.ISUDDEN)GO TO 172
      IF((REMAIND/ALAMDA).LT.0.5)EQ(ISUDDEN,LCASE)=
     +	   EQ(ISUDDEN,LCASE)-  ALAMDA*(RP(ISUDDEN)+RP(ISUDDEN+1))/2.
      IF((REMAIND/ALAMDA).GE.0.5)EQ(ISUDDEN,LCASE)=
     +	   EQ(ISUDDEN,LCASE)-	 ALAMDA*RP(ISUDDEN)
      GO TO 174
172   CONTINUE
      IF(I.NE.ISUDDEN+1)GO TO 175
      IF((REMAIND/ALAMDA).LT.0.5)EQ(ISUDDEN+1,LCASE)=
     +	   EQ(ISUDDEN+1,LCASE)- ALAMDA*RP(ISUDDEN+1)
      IF((REMAIND/ALAMDA).GE.0.5)EQ(ISUDDEN+1,LCASE)=
     +	   EQ(ISUDDEN+1,LCASE)- ALAMDA*(RP(ISUDDEN)+RP(ISUDDEN+1))/2.
      GO TO 174
175   CONTINUE
171   CONTINUE
      EQ(I,LCASE)=EQ(I,LCASE)-ALAMDA*(RP(I-1)+10.*RP(I)+
     +				      RP(I+1))/12.
174   CONTINUE
10    CONTINUE

      IF(IBB.EQ.0)GO TO 20
      EQ(1,LCASE)=EQ(1,LCASE)+CB/ALAMDA -F1r
      EQ(2,LCASE)=EQ(2,LCASE)-CB/ALAMDA
20    CONTINUE
      IF(IBB.EQ.0)EQ(1,LCASE)=EQ(1,LCASE)-F1r


      IF(IBT.EQ.0)GO TO 30
      EQ(100,LCASE)=EQ(100,LCASE)+CT/ALAMDA
      EQ(101,LCASE)=EQ(101,LCASE)-CT/ALAMDA -F101r
30    CONTINUE
      IF(IBT.EQ.0)EQ(101,LCASE)=EQ(101,LCASE)-F101r

      IF(XSUDDEN. EQ . 0.0) GO TO 111
C     Find out the number of the two nodes between which the sudden thick-
C     ness change occurs. Apply at the two nodes forces equivalent to the
C     temperature expansion restraining couple reversed in direction .

      ISUDDEN=IDINT(XSUDDEN/ALAMDA)+1

      AMSUDD=-(E*ALPHA*HSQDIF/(1.-PR))*
     +	    ( (TO(LCASE)-TI(LCASE)) /12.)

      EQ(ISUDDEN,LCASE)=EQ(ISUDDEN,LCASE)-(AMSUDD/ALAMDA)
      EQ(ISUDDEN+1,LCASE)=EQ(ISUDDEN+1,LCASE)+(AMSUDD/ALAMDA)
111   CONTINUE


      RETURN
      END


      SUBROUTINE PRESC(E,PR,R,ALAMDA,H,WB,THETAB,WT,THETAT,wPRESC,EF)

c     Deflections  ,  edge  reactions  and  bending  moments  due  to
c     prescribed edge displacements .


      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION AKPRESC(101,3),WP(101,4),FP(101,4),FM(4,4),EF(4)
      DIMENSION D(4),wPRESC(101)
      DIMENSION H(101)
      CALL STIFF(E,PR,R,ALAMDA,H,AKPRESC,XSUDDEN)
      DO 10 I=1,101
      DO 10 J=1,4
10    FP(I,J)=0.0

c     Apply a unit radial force or a unit moment at each edge.

      FP(1,1)=-1.0
      FP(1,2)=+1./ALAMDA
      FP(2,2)=-1./ALAMDA
      FP(101,3)=-1.0
      FP(100,4)=+1./ALAMDA
      FP(101,4)=-1./ALAMDA

      CALL EQSOLN(AKPRESC,FP,WP,101,3,4)

c     Generate flexibility matrix
      FM(1,1)=WP(1,1)
      FM(2,1)=(WP(2,1)-WP(1,1))/ALAMDA
      FM(3,1)=WP(101,1)
      FM(4,1)=(WP(101,1)-WP(100,1))/ALAMDA
      FM(2,2)=(1./(6.*ALAMDA))*
     +	      (-11.*WP(1,2)+18.*WP(2,2)-9.*WP(3,2)+2.*WP(4,2))
      FM(3,2)=WP(101,2)
      FM(4,2)=(WP(101,2)-WP(100,2))/ALAMDA
      FM(3,3)=WP(101,3)
      FM(4,3)=(WP(101,3)-WP(100,3))/ALAMDA
      FM(4,4)=-(1./(6.*ALAMDA))*
     +	      (-11.*WP(101,4)+18.*WP(100,4)-9.*WP(99,4)+2.*WP(98,4))

c     Rearrange flexibility matrix in the band form required for use of
c     the subroutine EQSOLN

      FM(1,2)=FM(2,1)
      FM(1,3)=FM(3,1)
      FM(1,4)=FM(4,1)
      FM(2,1)=FM(2,2)
      FM(2,2)=FM(3,2)
      FM(2,3)=FM(4,2)
      FM(2,4)=0.
      FM(3,1)=FM(3,3)
      FM(3,2)=FM(4,3)
      FM(3,3)=0.
      FM(3,4)=0.
      FM(4,1)=FM(4,4)
      FM(4,2)=0.
      FM(4,3)=0.
      FM(4,4)=0.

c     Adjust FM at the coordinate where the edge force is  required  to
c     be zero.

      IF(WB.EQ.0.0)FM(1,1)=FM(1,1)*10.E6
      IF(THETAB.EQ.0.0)FM(2,1)=FM(2,1)*10.E6
      IF(WT.EQ.0.0)FM(3,1)=FM(3,1)*10.E6
      IF(THETAT.EQ.0.0)FM(4,1)=FM(4,1)*10.E6

c     Arrange the prescribed edge displacements in vector D .

      D(1)=-WB
      D(2)=-THETAB
      D(3)=-WT
      D(4)=-THETAT


      CALL EQSOLN(FM,D,EF,4,4,1)

c     By superposition find deflections due to the edge forces EF.

      DO 30 I=1,101
      DO 30 J=1,4
30    wPRESC(I)=wPRESC(I)+WP(I,J)*EF(J)

      RETURN

      END

      SUBROUTINE LAGRANH(X1,X2,X3,H1,H2,H3,ALAMDA,H)
c     Calculate thickness at node points located between X1 and X3
c     by interpolation between three given thickness values. The
c     interpolation is done using Lagrange polonomial.
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION H(101)

      DO 10 I=1,101
      X=ALAMDA*(I-1)
      IF(X.LT.X1)GO TO 10
      IF(X.GT.X3*1.0000001)GO TO 10
      C1=((X-X2)/(X1-X2))*((X-X3)/(X1-X3))
      C2=((X-X1)/(X2-X1))*((X-X3)/(X2-X3))
      C3=((X-X1)/(X3-X1))*((X-X2)/(X3-X2))
      H(I)=C1*H1+C2*H2+C3*H3
10    CONTINUE
      RETURN
      END




c		 Solution of Banded Symmetrical Equations
c		      For Use in Structural Analysis
c		   Based on Ghali,A. and Neville,A. M.,
c      Structural Analysis :A Unified Classical and Matrix Approach
c	Chapman and Hall,London and New York,3 ed.,1990,Chapter 24
c----------------------------------------------------------------------
c		     Notation and dimensions of arrays
c							     rows  cols.
c    NB     band width ; see Sec. 24-3 of Ghali and Nevile.
c    NLC    number of load cases
c    [D]    nodal displacements 			     NDF    NLC
c    [F]    nodal restraining forces			     NDF    NLC
c    [S]    stiffness matrix				     NDF    NB
c-----------------------------------------------------------------------

c			  Program Listing
c
      SUBROUTINE EQSOLN(S,F,D,NDF,NB,NLC)
c     Solution of equilibrium equations [S][D]=-[F], using Cholesky's
c     method; see Sec. 25-11 of Ghali and Neville.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION S(NDF,NB),F(NDF,NLC),D(NDF,NLC)
      IF(S(1,1).LE. 0.0)GO TO 98
      DO 50 J=2,NDF
      JJ=J-1
      JJJ=J-NB+1
      IF(JJJ.LT.1)JJJ=1
      IF(JJ.EQ.1)GO TO 30
      DO 20 I=2,JJ
      II=I-1
      IF(II.LT.JJJ)GO TO 20
      SUM=S(I,J-I+1)
      DO 10 K=JJJ,II
10    SUM=SUM-S(K,I-K+1)*S(K,J-K+1)
      S(I,J-I+1)=SUM
20    CONTINUE
30    SUM=S(J,1)
      DO 40 K=JJJ,JJ
      T=S(K,J-K+1)/S(K,1)
      SUM=SUM-T*S(K,J-K+1)
40    S(K,J-K+1)=T
      IF(SUM.LE.0.0)GO TO  98
50    S(J,1)=SUM
c     At this stage,the first column of the band matrix [S] is replaced
c     by e11,e22,... and the remaining elements of the same matrix are
c     replaced by  elements uij using Eq. 24-49 to 24-53 of Ghali and
c     Neville. Now by forward substitution (Eq. 24-48), determine [g]
c     and store in [D].

      DO 95 L=1,NLC
      DO 70 I=1,NDF
      J=I-NB+1
      IF(I.LE.NB)J=1
      SUM=-F(I,L)
      KK=I-1
      IF(J.GT.KK)GO TO 65
      DO 60 K=J,KK
60    SUM=SUM-S(K,I-K+1)*D(K,L)
65    D(I,L)=SUM
70    CONTINUE
      DO 80 I=1,NDF
80    D(I,L)=D(I,L)/S(I,1)

c     By Eq. 24-46 of Ghali and Neville and backward substitution(Eq.24-
c     47),determine the required displacement values and store in [D].
      DO 95 II=1,NDF
      I=NDF-II+1
      J=I+NB-1
      IF(J.GT.NDF) J=NDF
      SUM=D(I,L)
      KK=I+1
      IF(KK.GT.J) GO TO 90
      DO 85 K=KK,J
85    SUM=SUM-S(I,K-I+1)*D(K,L)
90    D(I,L)=SUM
95    CONTINUE
      GO TO 99
98    WRITE(6,100)
100   FORMAT(/,3X,'Stiffness matrix is not positive definite')
      STOP
99    CONTINUE
      RETURN
      END


