$debug
c                  Computer Program SOR (Shell Of Revolution)  
c			      Version 1.0
c                              March 2013

c			     by A. Ghali
c                   Professor Emeritus,Civil Engineering
c			  The University of Calgary
c			   Calgary, Canada T2N 1N4

c		      Analysis of Axisymmetric Shells
c                           Based on Ghali,A.
c		      Circular Storage Tanks and Silos
c                         Third edition, Chapter 5
c------------------------------------------------------------------------------
c				 Scope
c     SHELLR analyses shells of revolution by the finite element method. The
c  element used has the shape of a frustum of a cone of constant thickness.
c  The idealized structure is defined by (x,r) coordinates of nodes situated
c  on a radial half section passing through the axis of revolution. The x-axis,
c  pointing uppwards, coincides with the axis of revolution. The analysis gives
c  gives the nodal displacements, the reactions and four stress resultants at
c  mid-height of each element. The stress resultants are Ns,Nphi,Ms,Mphi; where
c  N is normal force per unit length; M is bending moment per unit length; the
c  subscript s refers to direction of a mridian; the subscript phi refers to
c  circumpherential (hoop) direction.
c     The loading can be any, or cmbination,of the following types:
c      (a) Two concentrated forces and a couple, uniformly distributed over
c	   a nodal line. Enter load intensities:q1 (upwards),q2 (radial out-
c	   wards)and q3 (radial moment, repreesented by clockwise arrow on the
c	   right-hand half sectional elevation).
c      (b) Uniform load, whose intensities per unit area are qt and qn;
c	   where qt is tangential load in directed along a meridian, from
c	   starting node to end node; qn is normal to element surface pointing in direction of local coordinates 2* and 5*.
c	   in direction of local coordinates 2* and 5*.
c	   The self-weight of the structure can be determined by the computer.
c	   The command for this loading, which is a part of loading type(b),
c	   is indicated below.
c      (c) Temperature rise in elements; in which case the element
c	   number the temperatures Ti and To at the inner and outer
c	   surface are given. It is assumed that the rise of temperature
c	   varies linearly between Ti and To.
c
c			      Sign Convention
c     Positive displacement u is vertical upwards; positive displacement w is
c  horizontal radial outwards; positive rotation theta is represented by
c  clockwise arrow on the sectional elevation of the right-hand half of
c  sectional elevation of the structure. N is positive when tensile. M is
c  is positive when producing tension at the outer surface of the conical.
c  When the two nodes defining an element have the same x-coordinate, the
c  element has the shape of annular plane; positive moment produces tension at
c  the bottom face, provided that r1 is less than r2. The subscripts 1 and 2
c  refer to the first node and second node of the element. The reaction
c  components are determined in the directions of the global axes.

c
c			  Input Data Instructions
c     The data, in free format , must be in a file named SOR.IN .  
c  The data is composed of single lines or sets of lines as follows:
c    --Single line of title ,not exceeding 72 characters.
c    --Single line of 4 integers: number of nodes,number of elements,number of
c      joints with prescribed displacement(s) and number of load cases.
c    --Single line of three real values for E and Poisson's ratio and Mu ;
c      where E is elasticity modulus; Mu is thermal expansion coefficient.
c    --A set of lines ;their number equals number of nodes.Each line is comp-
c      osed of a node number followed by its x and r coordinates.
c    --A set of lines their number equals number of elements.Each line starts
c      with 3 integers: a member number ,its starting node and its end node,
c      followed by three real values: thickness, t, elasticity modulus, E and
c      Poisson's ratio..
c    --A set of lines their number equals number of joints with prescribed
c      displacements . Each line starts with a node number and three restraint
c      indicators (0, 1 or 2),followed by three real values of prescribed
c      displacements. The restraint indicators 1 and 0 mean free and prescribed
c      displacement component,respectively. When a displacement component is
c      free, a dummy value must be filled in for the prescribed displacement
c      (any real number). When the indicator is 2 it indicates spring support;
c      the corresponding real number on the same line is the spring stiffness
c      (force per length squared or force length per length)
c    --A set of any number of lines for loading of type(a).Each line is comp-
c      osed of two integers ,a load case number and a node number ,followed by
c      three real values of nodal forces q1,q2 and q3.The last line of the set
c      must start by an integer greater than the number of load cases follwed
c      by a dummy integer and three  dummy real numbers. The last line must not
c      be omitted even when there is no loading of type (a).
c    --A set of any number of lines for loading of type(b). Each line is comp-
c      osed of two integers, load case number and elemet number, followed by
c      two real values of intensities(per unit area) of tangential load,qt
c      pointing from starting node to end node, and normal load, qn in direct-
c      ion of local coordinates 2* and 5*. If the analysis is for the effect
c      of self weight of the structure is required, include in this group a
c      line composed of load case number, the integer zero, the weight per
c      unit volume and zero. The last line of this set must start  with an
c      integer greater than the number of load cases, followed by dummy
c      integer and two real values. The  last line must not be omitted even
c      when there is no loading of type (b).
c    --A set of any number of lines for loading of type(c).Each line is comp-
c      osed of two integers,a load case number and elemet number, followed by
c      two real values of temperature rise at the inner and outer surfaces. The
c      last line of this set must start with an integer greater than the number
c      of load cases followed by a dummy integer and six dummy real values. The
c      last line must not be omitted even when there is no loading of type (c).
c    --The integer 0 or 1; indicating longer or shorter output. The longer
c      includes  four tables that can be used to plot each stress resultant
c      on the generatrix of the shell as datum. (The generatrix is the line
c      that generates the shell surface by revolving  around the x-axis).
c      each table lists coordinates (r,x) for two lines: the generatrix and
c      and a graph representing the value of one of the stress resultants.
                
c    Examle input data:

c   Example 2-1, Ghali,Tanks and Silos, cylinderical wall hydr. ld.
c   11	  10	1    1			NJ,NE,NSJ,NLC
c   0.                              Thermal expansion coefficient
c   1    0.00         3.0           Node no., x and r coordinates.
c   2	 0.05	      3.0
c   3	 0.10	      3.0
c   4	 0.15	      3.0
c   5	 0.20	      3.0
c   6	 0.25	      3.0
c   7	 0.40	      3.0
c   8	 0.55	      3.0
c   9	 0.70	      3.0
c   10	 0.85	      3.0
c   11	 1.00	      3.0
c   1    1  2  0.0667  1.0  0.167 Element no.,JS,JE,thickness, E, Poisson's r.
c   2    2  3  0.0667  1.0  0.167
c   3    3  4  0.0667  1.0  0.167
c   4    4  5  0.0667  1.0  0.167
c   5    5  6  0.0667  1.0  0.167
c   6    6  7  0.0667  1.0  0.167
c   7    7  8  0.0667  1.0  0.167
c   8    8  9  0.0667  1.0  0.167
c   9    9 10  0.0667  1.0  0.167
c  10   10 11  0.0667  1.0  0.167
c   1  0  0  1   0.0   0.0    0.0   Node no.,restraint indics.,prsc. displs.
c   1    1  .0   0.024583 .0 Ld. case,node,F1(up),F2(horiz. out),F3(rad. mt.)
c   1    2  .0    0.0475   .0
c   1    3  .0    0.0450   .0
c   1    4  .0    0.0425   .0
c   1    5  .0    0.0400   .0
c   1    6  .0    0.0717   .0
c   1    7  .0    0.0900   .0
c   1    8  .0    0.0675   .0
c   1    9  .0    0.0450   .0
c   1   10  .0    0.0225   .0
c   1   11  .0    0.00375  .0
c  10	 0  0.	  0.	0.    Dummy,end of data of loaded nodes
c   1  1  0. 0.  Ld. case,element , qt(tangential),qn(normal)(force/unit area)
c   1  1  0. 0.95
c   1  2  0. 0.85
c   1  3  0. 0.75
c   1  4  0. 0.65
c   1  5  0. 0.55
c   1  6  0. 0.45
c   1  7  0. 0.35
c   1  8  0. 0.25
c   1  9  0. 0.15
c   1  10 0. 0.05
c  10  0  0.  0.     Dummy,end end of data of loaded elements
c   1    1  0.    0. Case,element,Ti,To(temp. inner& outer face)
c  10	 0  0.	  0.	      Dummy,end thermal load data
c   0                Integer 1 or 0.0,  for long or short output, repectively.

c    The letter c at the start of each data line  must ,of course , be omitted.
c  To run the program , type PLANEF and press Enter. The program will execute
c  the analysis and give the input and the results in a file named PLANEF.OUT .

c			     Limits on Data
c    Vectors {B} and {NI} are used to store all the real and the integer arrays
c  used by the program. The maximum dimensions set are: MAXB=20000 and MAXNI=
c  2000 . This would normally be sufficient for approximately 120 nodes, 120
c  elements and 8 load cases, depending upon the band width. The maximum dim-
c  ensions can be altered by changing the numbers on line 2 of fortran listing.
c  After the change, the program and EQSOLN.FOR (equation solution subroutine)
c  must be compiled and linked using a Fortran compiler(e.g. Microsoft Fortran
c  Optimizing compiler).

c------------------------------------------------------------------------------
c		     Notation and dimensions of arrays
c								     rows  cols.
c  [Ar]   thermal stress resultants  with nodal displacements	    4	 NRAR
c	  restrained,for all elements and for all cases of loading.
c  [D]	  nodal displacements					    3*NJ   NLC
c   E	   modulus of elasticity
c  [ES]   element  stiffness in local coordinates		     6	    6
c  [EST]  element stiffness matrix in directions parallel to	     6	    6
c	  global axes
c  [F]	  nodal restraining forces				    3*NJ   NLC
c  [ISUP] joint restraint indicators : 1 and 0 ,indicate	     NSJ    3
c	  and prescribed displacement, respectively; 2 indicates
c	  spring support.
c  {JE}   end nodes of individual members			     NM
c  {JS}   starting node of individual members			     NM
c  [LR]   list of load case and member number for which temperature  2	  NRAR
c	  data are given
c  NB	  band width ; see Sec. 24-3 of Ghali and Neville
c  NJ	  number of joints (nodes)
c  NLC	  number of load cases
c  NE	  number of members
c  NRAR   number of rows in [AR] ; this is equal the number of mem-
c	  bers for which temperature data are given as data
c  {NS}   list of node numbers where supports are provided or disp-  NSJ
c	  lacements are prescribed
c  NSJ	  number of supprted joints (nodes with prescribed displac-
c	  ements)
c  [PD]   prescribed displacement values, or spring stiffness	     NSJ   3
c  PR	  Poisson's ratio
c  [R]	  forces at the supprted nodes including the reactions	    3*NSJ NLC
c  [S]	  stiffness matrix					    3*NJ   NB
c  {SIGMAR} thermal stress resultants  with nodal displacements,     4
c	  for individual element
c  {SK}   diagonal elements of [S] corresponding to  coordinates    3*NSJ
c	  at the supportd nodes
c  {t}	  element thicknesses					      NE
c  {X}	  x coordinate of nodes 				      NJ
c  {r}	  r coordinates of nodes
c------------------------------------------------------------------------------

c		 Listing of program SHELLR
c
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER(MAXB=60000,MAXNI=6000)
      COMMON/REALV/B(MAXB)
      COMMON/NINTGR/NI(MAXNI)
      COMMON/STDAT1/NJ,NE,NSJ,NRAR
      CHARACTER * 76 TITLE
      OPEN(5,FORM='FORMATTED',MODE='READ',FILE='SOR.IN')
      OPEN(6,FORM='FORMATTED',MODE='WRITE',FILE='SOR.OUT')
      DO 10 I=1,MAXB
10    B(I)=0.0
      DO 20 I=1,MAXNI
20    NI(I)=0
      READ(5,100)TITLE
100   FORMAT(A76)
      WRITE(6,200)TITLE
200   FORMAT(3X,A76)
      READ(5,*)NJ,NE,NSJ,NLC
      WRITE(6,300)NJ
300   FORMAT(3X,'Number of joints=',I5)
      WRITE(6,400)NE
400   FORMAT(3X,'Number of elements=',I5)
      WRITE(6,500)NSJ
500   FORMAT(3X,'Number of joints with prescribed displacement(s)=',I5)
      WRITE(6,600)NLC
600   FORMAT(3X,'Number of load cases=',I5)
      NJ3=3*NJ
      NS3=3*NSJ

c     Elements of {B} at which real arrays start are:
c     {t}:N1  [D]:N2  [F]:N3  [PD]:N4  [RR]:N5	{SK}:N6
c     {X}:N7  {r}:N8  {E}:N9  {Pr}:N10	[S]:N11  [AR]:N12
      N1=1
      N2=N1+NE
      N3=N2+NJ3*NLC
      N4=N3+NJ3*NLC
      N5=N4+NS3
      N6=N5+NS3*NLC
      N7=N6+NS3
      N8=N7+NJ
      N9=N8+NJ
      N10=N9+NE
      N11=N10+NE

c     Elements of {N} at which integer arrays start are:
c     {JS}:K1  {JE}:K2	{NS}:K3  [ISUP]:K4  [LR]:K5
      K1=1
      K2=K1+NE
      K3=K2+NE
      K4=K3+NSJ
      K5=K4+NS3

c     CALL DATASHELL(X,r,JS,JE,t,NS,ISUP,PD,NB,ALPHA,E,Pr)
      CALL DATASHELL(B(N7),B(N8),NI(K1),NI(K2),B(N1),NI(K3),
     *NI(K4),B(N4),NB,ALPHA,B(N9),B(N10))
c     Identification of elements of {B} at which real arrays start
c     (continued)
      N12=N11+NJ3*NB

c     Determine maximum number of rows of [AR] that can be stored in
c     the remaining space in {B}. Also determine the maximum number of
c     rows of [LR] which can be stored in the remaining space in {NI}.
      NAR=(MAXB-N12)/4
      NLR=(MAXNI-K5)/2

c     CALL STIFSHELL(JS,JE,t,X,r,S,NJ3,NB,E,Pr)
      CALL STIFSHELL(NI(K1),NI(K2),B(N1),B(N7),B(N8),B(N11),
     *NJ3,NB,B(N9),B(N10))

c     CALL LOADSHELL(JS,JE,X,r,F,LR,AR,NJ3,NLR,NAR,NLC,t,ALPHA,E,Pr,
C    * IOUTPUT)
      CALL LOADSHELL(NI(K1),NI(K2),B(N7),B(N8),B(N3),NI(K5),B(N12),NJ3,
     *NLR,NAR,NLC,B(N1),ALPHA,B(N9),B(N10),IOUTPUT)

c     CALL SUPTSHELL(S,F,NS,ISUP,PD,RR,SK,NJ3,NS3,NB,NLC,r)
      CALL SUPTSHELL(B(N11),B(N3),NI(K3),NI(K4),B(N4),B(N5),B(N6),NJ3,
     *NS3,NB,NLC,B(N8))

c     CALL EQSOLN(S,F,D,NJ3,NB,NLC)
      CALL EQSOLN(B(N11),B(N3),B(N2),NJ3,NB,NLC)

c     CALL RSLTSHELL(D,NS,ISUP,PD,RR,SK,X,r,JS,JE,t,LR,AR,NLR,
c    *NAR,NJ3,NS3,NLC,E,Pr,IOUTPUT)
      CALL RSLTSHELL(B(N2),NI(K3),NI(K4),B(N4),B(N5),B(N6),B(N7),B(N8),
     *NI(K1),NI(K2),B(N1),NI(K5),B(N12),NLR,NAR,NJ3,NS3,NLC,
     *B(N9),B(N10),IOUTPUT)
      CLOSE(5)
      CLOSE(6)
      STOP
      END

      SUBROUTINE DATASHELL(X,r,JS,JE,t,NS,ISUP,PD,NB,ALPHA,E,Pr)
c     Reading data for plane frame (excluding loads)and calculation of
c     band width.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NE,NSJ,NRAR
      DIMENSION X(NJ),r(NJ),JS(NE),JE(NE),t(NE),E(NE),Pr(NE)
      DIMENSION NS(NSJ),ISUP(NSJ,3),PD(NSJ,3)
      READ(5,*)ALPHA
      WRITE(6,200)ALPHA
200   FORMAT(/,3X,'Coefficient of thermal expansion=',E13.5)
      WRITE(6,300)
300   FORMAT(/,3X,'Nodal coordinates')
      WRITE(6,400)
400   FORMAT(5X,'Node',5x,'x',9X,'r')
      DO 10 I=1,NJ
      READ(5,*)J,X(J),r(J)
10    WRITE(6,500)J,X(J),r(J)
500   FORMAT(3X,I5,2F10.3)
      WRITE(6,600)
600   FORMAT(/,3X,'Element information')
      WRITE(6,700)
700   FORMAT(3X,'Element',2X,'1st node',3X,'2nd node',6x,'t',10X,
     +'E-modulus',3X,'Poisson s r.')
      DO 20 I=1,NE
      READ(5,*)J,JS(J),JE(J),t(J),E(J),Pr(J)
c     Member no.,starting node,end node,thickness,E-modulus,Poisson's r.
20    WRITE(6,800)J,JS(J),JE(J),t(J),E(J),Pr(J)
800   FORMAT(3X,I5,4X,I5,5X,I5,3X,3(E13.5,1X))
      WRITE(6,900)
900   FORMAT(/,3X,'Support conditions')
      WRITE(6,1000)
1000  FORMAT(5X,
     *'Node',5X,'Restraint indicators',7X,'Prescribed displacements',
     +/,3X,
     +'                                       or spring stiffnesses')
c     Restraint indicators are 1 or 0,meaning free or prescribed disp-
c     lacement,respectively. Indicator 2 means spring support.
      WRITE(6,1100)
1100  FORMAT(16x,'u',6x,'w',3X,'theta',8x,'u',13x,'w',12x,'theta')
1200  FORMAT(3X,I5,4X,I5,2X,I5,2X,I5,3(2X,E13.5))
      DO 30 J=1,NSJ
      READ(5,*)NS(J),ISUP(J,1),ISUP(J,2),ISUP(J,3),PD(J,1),PD(J,2),PD(J,
     *3)
30    WRITE(6,1200)NS(J),ISUP(J,1),ISUP(J,2),ISUP(J,3),PD(J,1),PD(J,2),
     *PD(J,3)
      WRITE(6,31)
31    FORMAT(/,3X,
     +'The above table gives the prescribed displacement value or the',/
     +,3X,'spring stiffness when the indictor is 0 or 2,respectively.')
c     Determine band width by Eq. 24-1 of Ghali and Neville.
      NBI=0
      DO 50 I=1,NE
      N=JS(I)-JE(I)
      IF(N.GE.0) GO TO 40
      N=-N
40    CONTINUE
      IF(N.GT.NBI)NBI=N
50    CONTINUE
      NB=3*(NBI+1)
      RETURN
      END

      SUBROUTINE STIFSHELL(JS,JE,t,X,r,S,NJ3,NB,E,Pr)
c     Stiffness matrix for shell of revolution.See Chapter 5,Ghali
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NE,NSJ,NRAR
      DIMENSION JS(NE),JE(NE),t(NE),X(NJ),r(NJ),E(NE),Pr(NE)
      DIMENSION S(NJ3,NB),GAUSS(2,2)
      DIMENSION ES(6,6),EST(6,6),IM(6)
      N3=3*NJ
      DO 10 I=1,N3
      DO 10 J=1,NB
      S(I,J)=0.0
10    CONTINUE
      CALL GAUSS2(GAUSS)
      DO 50 JM=1,NE
c     Member stiffness matrix in local coordinates,considering shear
c     deformation.
c     CALL ELSTIFF(r1,x1,r2,x2,t,ES,c,sin,GAUSS,E,Pr)
      CALL ELSTIFF(r(JS(JM)),
     +X(JS(JM)),r(JE(JM)),X(JE(JM)),t(JM),ES,c,sin,GAUSS,E(JM),Pr(JM))
c     Transform element stiffness,[ES] into [EST] in global directions;
      CALL TRANSFORMES(c,sin,ES,EST)
c     Identify the six coordinates at the start and end of member.
      IM(1)=3*JS(JM)-2
      IM(2)=IM(1)+1
      IM(3)=IM(1)+2
      IM(4)=3*JE(JM)-2
      IM(5)=IM(4)+1
      IM(6)=IM(4)+2
      DO 40 I=1,6
      DO 40 J=1,6
      K=IM(J)-IM(I)
      IF(K.LT.0)GO TO 35
      K=K+1
      S(IM(I),K)=S(IM(I),K)+EST(I,J)
35    CONTINUE
40    CONTINUE
50    CONTINUE
      RETURN
      END

      SUBROUTINE LOADSHELL(JS,JE,X,r,F,LR,AR,NJ3,NLR,NAR,NLC,t,ALPHA,
     +E,Pr,IOUTPUT)
c     Generation of vectors of restraining forces,[F]
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NE,NSJ,NRAR
      DIMENSION JS(NE),JE(NE),X(NJ),r(NJ),F(NJ3,NLC),LR(2,NLR)
      DIMENSION AR(4,NAR),SIGMAR(4),DE(4,4),FRT(6),t(NE),GAUSS(2,2)
      DIMENSION E(NE),Pr(NE)
      Pi=3.141592654
      WRITE(6,100)
100   FORMAT(/,3X,'Forces applied at the nodes')
      WRITE(6,200)
200   FORMAT(3X,'Load case ',3X,'Node',7X,'Fx',12X,'Fr',12X,'M')
20    CONTINUE
      READ(5,*)LCASE,NODE,FX,Fr,AMZ
c     IF(LCASE.GT.NLC)GO TO 30
      WRITE(6,300)LCASE,NODE,FX,Fr,AMZ
300   FORMAT(3X,I5,6X,I5,3(1X,E13.5))
      IF(LCASE.GT.NLC)GO TO 30
      N1=3*NODE-2
      N2=N1+1
      N3=N2+1
      F(N1,LCASE)=F(N1,LCASE)-FX *(2.*Pi*r(NODE))
      F(N2,LCASE)=F(N2,LCASE)-Fr *(2.*Pi*r(NODE))
      F(N3,LCASE)=F(N3,LCASE)-AMZ*(2.*Pi*r(NODE))
      GO TO 20
30    CONTINUE
c     Distributed load data
      ISELFWT=0
      WRITE(6,110)
110   FORMAT(/,3X,
     +'Elements subjected to distributed load,and load per unit area')
      WRITE(6,120)
120   FORMAT(3X,
     +'Load case ',2X,'Element',4X,'Tangential ld.',2X,'Normal ld.')
35    CONTINUE
      READ(5,*)LCASE,NELEMENT,qt,qn
      WRITE(6,130)LCASE,NELEMENT,qt,qn
130   FORMAT(3X,I5,6X,I5,6X,E13.5,2X,E13.5)
      IF(LCASE.GT.NLC)GO TO 27
      IF(NELEMENT.EQ.0)GO TO 24
      Alr=r(JE(NELEMENT))-r(JS(NELEMENT))
      Alx=X(JE(NELEMENT))-X(JS(NELEMENT))
      AL=DSQRT(ALr*Alr+Alx*Alx)
      sin=Alr/AL
      c=Alx/AL
      N1=3*JS(NELEMENT)-2
      N2=N1+1
      F(N1,LCASE)=F(N1,LCASE)-(Pi*AL/3.)*
     +			(  c*qt*(2.*r(Js(NELEMENT))+r(JE(NELEMENT)))
     +			-sin*qn*(2.*r(Js(NELEMENT))+r(JE(NELEMENT))) )
      F(N2,LCASE)=F(N2,LCASE)-(Pi*AL/3.)*
     +			(sin*qt*(2.*r(Js(NELEMENT))+r(JE(NELEMENT)))
     +			+  c*qn*(2.*r(Js(NELEMENT))+r(JE(NELEMENT))) )
      N1=3*JE(NELEMENT)-2
      N2=N1+1
      F(N1,LCASE)=F(N1,LCASE)-(Pi*AL/3.)*
     +			(  c*qt*(r(Js(NELEMENT))+2.*r(JE(NELEMENT)))
     +			-sin*qn*(r(Js(NELEMENT))+2.*r(JE(NELEMENT))) )
      F(N2,LCASE)=F(N2,LCASE)-(Pi*AL/3.)*
     +			(sin*qt*(r(Js(NELEMENT))+2.*r(JE(NELEMENT)))
     +			+  c*qn*(r(Js(NELEMENT))+2.*r(JE(NELEMENT))) )
      GO TO 26
24    CONTINUE
      IF(NELEMENT.EQ.0)GAMMA=qt
      IF(NELEMENT.EQ.0)ISELFWT=LCASE
      DO 25 NELT=1,NE
      Alr=r(JE(NELT))-r(JS(NELT))
      Alx=X(JE(NELT))-X(JS(NELT))
      AL=DSQRT(ALr*Alr+Alx*Alx)
      sin=Alr/AL
      c=Alx/AL
      qt=-GAMMA*t(NELT)*c
      qn= GAMMA*t(NELT)*sin
      N1=3*JS(NELT)-2
      N2=N1+1
      F(N1,LCASE)=F(N1,LCASE)-(Pi*AL/3.)*
     +			(  c*qt*(2.*r(Js(NELT))+r(JE(NELT)))
     +			-sin*qn*(2.*r(Js(NELT))+r(JE(NELT))) )
      F(N2,LCASE)=F(N2,LCASE)-(Pi*AL/3.)*
     +			(sin*qt*(2.*r(Js(NELT))+r(JE(NELT)))
     +			+  c*qn*(2.*r(Js(NELT))+r(JE(NELT))) )
      N1=3*JE(NELT)-2
      N2=N1+1
      F(N1,LCASE)=F(N1,LCASE)-(Pi*AL/3.)*
     +			(  c*qt*(r(Js(NELT))+2.*r(JE(NELT)))
     +			-sin*qn*(r(Js(NELT))+2.*r(JE(NELT))) )
      F(N2,LCASE)=F(N2,LCASE)-(Pi*AL/3.)*
     +			(sin*qt*(r(Js(NELT))+2.*r(JE(NELT)))
     +			+  c*qn*(r(Js(NELT))+2.*r(JE(NELT))) )
25    CONTINUE
26    CONTINUE
      GO TO 35
27    CONTINUE
      IF(ISELFWT.NE.0)WRITE(6,140)ISELFWT,GAMMA
140   FORMAT(/,3X,'Self weight is included in case ',I2,
     +		 '. Weight per unit volume=',E12.5)
c     Thermal load data
      WRITE(6,400)
400   FORMAT(/,3X,'Elements subjected to temperature change')
      WRITE(6,500)
500   FORMAT(2X,'Ld. case ',1x,'Element',2X,'Temperature rise at',/,3X,
     +		'                 inner face    outer face')
      NRAR=0
c     NRAR is number of rows in [AR]; that is equal to the number of
c     lines for loading of type (b).The last line of data is not inc-
c     luded.
      CALL GAUSS2(GAUSS)
40    CONTINUE
      NRAR=NRAR+1
      N=NRAR
      READ(5,*)LR(1,N),LR(2,N),Ti,To
      WRITE(6,600)LR(1,N),LR(2,N),Ti,To
600   FORMAT(3X,I5,1X,I5,7X,E10.4,1x,E10.4)
      IF(LR(1,N).GT.NLC)GO TO 50
      LCASE=LR(1,N)
      MEMBER=LR(2,N)
      CALL ELASTICITY(t(MEMBER),DE,E(MEMBER),Pr(MEMBER))
      CALL SIGMAREST(t(MEMBER),DE,ALPHA,Ti,To,SIGMAR)
      DO 42 L=1,4
42    AR(L,N)=SIGMAR(L)
      CALL TEMPFORCE
     +(r(JS(MEMBER)),x(JS(MEMBER)),r(JE(MEMBER)),x(JE(MEMBER)),t(MEMBER)
     +,Ti,To,ALPHA,FRT,GAUSS,E(MEMBER),Pr(MEMBER))
c     Identify the nodes and coordinate numbers at the member ends.
      N1=3*JS(MEMBER)-2
      N2=N1+1
      N3=N1+2
c     F(N1,LCASE)=F(N1,LCASE)+FRT(1)*2.*Pi*r(JS(Member))
c     F(N2,LCASE)=F(N2,LCASE)+FRT(2)*2.*Pi*r(JS(Member))
c     F(N3,LCASE)=F(N3,LCASE)+FRT(3)*2.*Pi*r(JS(Member))
      F(N1,LCASE)=F(N1,LCASE)+FRT(1)
      F(N2,LCASE)=F(N2,LCASE)+FRT(2)
      F(N3,LCASE)=F(N3,LCASE)+FRT(3)
      N1=3*JE(MEMBER)-2
      N2=N1+1
      N3=N1+2
c     F(N1,LCASE)=F(N1,LCASE)+FRT(4)*2.*Pi*r(JE(Member))
c     F(N2,LCASE)=F(N2,LCASE)+FRT(5)*2.*Pi*r(JE(Member))
c     F(N3,LCASE)=F(N3,LCASE)+FRT(6)*2.*Pi*r(JE(Member))
      F(N1,LCASE)=F(N1,LCASE)+FRT(4)
      F(N2,LCASE)=F(N2,LCASE)+FRT(5)
      F(N3,LCASE)=F(N3,LCASE)+FRT(6)
      GO TO 40
50    CONTINUE
      NRAR=NRAR-1

      WRITE(6,7000)
7000  FORMAT(/,3X,'The analysis includes calculation of nodal forces tha
     *t can artificially',/,3X,'prevent displacements due to all loads.'
     *,/)

      DO 60 J=1,NLC
      WRITE(6,700)J
      DO 60 I=1,NJ

C      C=2*3.141592654*r(J)   Mistake: The letter J should be I.
C Note: When r(I)= 0., the nodal force per unit length cannot be calculated.

      C=2*3.141592654*r(I)
      IF(C.NE.0.)
     *WRITE(6,800)I,(F((3*I-2),J)/C),(F((3*I-1),J)/C),(F((3*I),J)/C)
60    CONTINUE
700   FORMAT(/,3X,'Total restraining forces per unit length of nodal lin
     +es in load case',I2,':',/,3X,
     +'Node',5X,5X,'Fx',5X,2X,6X,'Fr',5X,2X,4X,'Moment')
800   FORMAT(3X,I4,3X,3(E13.5,2X))

      READ(5,*) IOUTPUT
c     IOUTPUT= 1 or 0, for longer or shorter output, respectively.The longer
c     output includes a table for each stress resultant, usable
c     for plotting two x-r lines representing the shell and the value of
c     the stress resultant on te same graph.

      RETURN
      END

      SUBROUTINE SUPTSHELL(S,F,NS,ISUP,PD,RR,SK,NJ3,NS3,NB,NLC,r)
c     Adjust [S] and [F] in accordance with the support conditions.See
c     Eq. 24-39 of Ghali and Neville.Note that before the adjustment,
c     [S] generated by the subroutine STIFPF is the stiffness matrix of
c     a free unsupported structure.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NE,NSJ,NRAR
      DIMENSION S(NJ3,NB),F(NJ3,NLC),NS(NSJ),ISUP(NSJ,3),PD(NSJ,3)
      DIMENSION RR(NS3,NLC),SK(NS3),r(NJ)
      Pi=3.141592654
      DO 10 I=1,NS3
      SK(I)=0.0
      DO 10 J=1,NLC
10    RR(I,J)=0.0
c     Use {SK} and [R] to store appropriate elements on the diagonal of
c     [S] and of [F] for use in calculation of the reactions by Eq. 24-
c     38 of Ghali and Neville.
      DO 50 I=1,NSJ
      M1=3*I-3
      N1=3*NS(I)-3
      DO 40 K=1,3
      IF(ISUP(I,K).EQ.1)GO TO 30
      M=M1+K
      N=N1+K
      SK(M)=S(N,1)
      IF(ISUP(I,K).EQ.0)S(N,1)=S(N,1)*1.0E6
      IF(ISUP(I,K).EQ.2)S(N,1)=S(N,1)+PD(I,K)*2.*Pi*r(NS(I))
      DO 20 J=1,NLC
      IF(ISUP(I,K).EQ.0)RR(M,J)=F(N,J)
      IF(ISUP(I,K).EQ.0)F(N,J)=-SK(M)*1.0E6*PD(I,K)
20    CONTINUE
30    CONTINUE
40    CONTINUE
50    CONTINUE
      RETURN
      END

      SUBROUTINE RSLTSHELL(D,NS,ISUP,PD,RR,SK,X,r,JS,JE,t,LR,AR,NLR,
     *NAR,NJ3,NS3,NLC,E,Pr,IOUTPUT )
c     Printing of nodal displacement reactions and member end forces.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NE,NSJ,NRAR
      DIMENSION D(NJ3,NLC),NS(NSJ),ISUP(NSJ,3),PD(NSJ,3)
      DIMENSION RR(NS3,NLC)
      DIMENSION SK(NS3),X(NJ),r(NJ),JS(NE),JE(NE),t(NE),E(NE),Pr(NE)
      DIMENSION LR(2,NLR),AR(4,NAR)
      DIMENSION DT(6),BB(4,6),DE(4,4),PRODUCT(4,6),SIGMA(4)
      DIMENSION PLNS(NE,4),PLNPHI(NE,4),PLMS(NE,4),PLMPHI(NE,4)
      DIMENSION PLOT1(NE,4),PLOT2(NE,4)
      WRITE(6,1000)
1000  FORMAT(3X,'------------------------------------------------------
     *---------------------')
      WRITE(6,1100)
1100  FORMAT(3X,'Analysis results')
      DO 60 K=1,NLC
      DO 6 II=1,NE
      DO 6 JJ=1,4
      PLNS(II,JJ)=0.
      PLNPHI(II,JJ)=0.
      PLMS(II,JJ)=0.
      PLMPHI(II,JJ)=0.
6     CONTINUE
      WRITE(6,100)K
100   FORMAT(/,3X,'Load case no.',I5)
      WRITE(6,200)
200   FORMAT(/,3X,'Nodal displacements')
      WRITE(6,300)
300   FORMAT(5X,'Node',9X,'u',15X,'w',13x,'theta')
      DO 10 I=1,NJ
      N1=3*I-2
      N2=N1+1
      N3=N1+2
      WRITE(6,400)I,D(N1,K),D(N2,K),D(N3,K)
400   FORMAT(3X,I5,3(3X,E13.5))
10    CONTINUE
      WRITE(6,500)
500   FORMAT(/,3X,'Forces per unit length of supported nodal lines')
      WRITE(6,600)
600   FORMAT(5X,'NODE',11X,'Fx',17X,'Fr',18X,'M')
      DO 30 L=1,NSJ
      M1=3*L-3
      N1=3*NS(L)-3
      DO 20 J=1,3
      IF(ISUP(L,J).EQ.1)GO TO 20
      M=M1+J
      N=N1+J
      IF(ISUP(L,J).EQ.0)THEN
      RR(M,K)=RR(M,K)+SK(M)*1.0E6*(PD(L,J)-D(N,K))+SK(M)*D(N,K)
      Pi=3.141592654
      RR(M,K)=RR(M,K)/(2.*Pi*r(NS(L)))
      END IF
      IF(ISUP(L,J).EQ.2)RR(M,K)=-D(N,K)*PD(L,J)
20    CONTINUE
      M1=M1+1
      M2=M1+1
      M3=M1+2
      WRITE(6,400)NS(L),RR(M1,K),RR(M2,K),RR(M3,K)
30    CONTINUE
      WRITE(6,700)
700   FORMAT(/,3X,
     +'Stress resultants in each element midway between its nodes')
      WRITE(6,800)
800   FORMAT(3X,
     +'Elem.',5X,'r',11X,'x',10X,'Ns',10X,'Nphi',10x,'Ms',8x,'Mphi')
      DO 50 JM=1,NE
      DO 51 MM=1,4
      DO 51 JJ=1,6
      PRODUCT(MM,JJ)=0.
51    SIGMA(MM)=0.
      ALX=X(JE(JM))-X(JS(JM))
      ALr=r(JE(JM))-r(JS(JM))
      ALJ=DSQRT(ALX*ALX+ALr*ALr)
      C=ALX/ALJ
      SIN=ALr/ALJ
      rmiddle=(r(JE(JM))+r(JS(JM)))/2.
      xmiddle=(x(JE(JM))+x(JS(JM)))/2.
      N1=3*JS(JM)-2
      N2=N1+1
      N3=N1+2
      N4=3*JE(JM)-2
      N5=N4+1
      N6=N4+2
c     Transform displacements at member ends into local coordinates.
      DT(1)=D(N1,K)*C+D(N2,K)*SIN
      DT(2)=-D(N1,K)*SIN+D(N2,K)*C
      DT(3)=D(N3,K)
      DT(4)=D(N4,K)*C+D(N5,K)*SIN
      DT(5)=-D(N4,K)*SIN+D(N5,K)*C
      DT(6)=D(N6,K)
c     Generate the B-matrix at middle
      EXI=0.5
      CALL Bmatrix(c,sin,ALJ,EXI,rmiddle,BB)
      CALL ELASTICITY(t(JM),DE,E(JM),Pr(JM))
      CALL MULTIPLY(4,4,6,DE,BB,PRODUCT)
      CALL MULTIPLY(4,6,1,PRODUCT,DT,SIGMA)
      IF(NRAR.EQ.0)GO TO 45
c     Add thermal stress resultants with displacements restrained.
      DO 40 I=1,NRAR
      IF(LR(1,I).NE.K) GO TO 35
      IF(LR(2,I).NE.JM) GO TO 35
      DO 41 L=1,4
41    SIGMA(L)=SIGMA(L)+AR(L,I)
35    CONTINUE
40    CONTINUE
45    CONTINUE
      PLOT1(JM,1)=rmiddle
      PLOT1(JM,2)=xmiddle
      PLOT1(JM,3)=c
      PLOT1(JM,4)=SIN
      PLOT2(JM,1)=SIGMA(1)
      PLOT2(JM,2)=SIGMA(2)
      PLOT2(JM,3)=SIGMA(3)
      PLOT2(JM,4)=SIGMA(4)
      WRITE(6,900)JM,rmiddle,xmiddle,SIGMA(1),SIGMA(2),SIGMA(3),SIGMA(4)
900   FORMAT(3X,I3,1X,6(1X,E11.5))
50    CONTINUE
      CALL PLOTDATA
     +(PLNS,PLNPHI,PLMS,PLMPHI,PLOT1,PLOT2,NE,IOUTPUT)

      IF(IOUTPUT.EQ.0)WRITE(6,1001)
1001  FORMAT(3X,'------------------------------------------------------
     *---------------------')


      IF(IOUTPUT.EQ.0)GO TO 610
      WRITE(6,901)
901   FORMAT(/,3X,
     +'Data for plotting the structure and Ns on same graph',/,3X,
     +'Coordinate r  Coordinate x     r for Ns       x for Ns' )
      WRITE(6,902)((PLNS(I,J),J=1,4),I=1,NE)
902   FORMAT(3X,E12.5,1X,E12.5,3X,E12.5,3X,E12.5)
      WRITE(6,903)
903   FORMAT(/,3X,
     +'Data for plotting the structure and Nphi on same graph',/,3X,
     +'Coordinate r  Coordinate x    r for Nphi     x for Nphi' )
      WRITE(6,902)((PLNPHI(I,J),J=1,4),I=1,NE)
      WRITE(6,904)
904   FORMAT(/,3X,
     +'Data for plotting the structure and Ms on same graph',/,3X,
     +'Coordinate r  Coordinate x     r for Ms       x for Ms' )
      WRITE(6,902)((PLMS(I,J),J=1,4),I=1,NE)
      WRITE(6,905)
905   FORMAT(/,3X,
     +'Data for plotting the structure and Mphi on same graph',/,3X,
     +'Coordinate r  Coordinate x    r for Mphi     x for Mphi' )
      WRITE(6,902)((PLMPHI(I,J),J=1,4),I=1,NE)
      WRITE(6,1200)
1200  FORMAT(3X,'-------------------------------------------------------
     *------------------------------------------------')

610   CONTINUE
60    CONTINUE
      RETURN
      END

      SUBROUTINE ELSTIFF(r1,x1,r2,x2,t,ES,c,sin,GAUSS,E,Pr)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
c     Stiffness matrix of conical shell element. Reference Zienkievicz second
c     ed., The Finite Element Method in Engineering Science, McGraw-Hill
c     1971, Sections 12.1 and 12.2. Equations revised by A. Ghali.
c     r1=radius at bottom edge; r2=radius at top edge; x axis, pointing up
c     is the symmetry axis of revolution x1 and x2= cordinates of bottom and
c     top edges; t=element thickness; E=modulus of elasticity; PR= Poisson's
c     ratio.
      COMMON/STDAT1/NJ,NE,NSJ,NRAR
      DIMENSION ES(6,6),BB(4,6),DE(4,4)
      DIMENSION BT(6,4),TEMP(4,6),SS(6,6)
      DIMENSION GAUSS(2,2)
      DO 5 I=1,6
      DO 5 J=1,6
5     ES(I,J)=0.
      Pi=3.141592654
c     DE is elasticity matrix. TEMP and SS are matrices for temporary storage
c     of matrix products. BB is matix relating generalized strains to nodal
c     displacements.
      CALL ELASTICITY(t,DE,E,pr)
      AL=DSQRT((r1-r2)**2+(x1-x2)**2)
      r=r1*(1.-EXI)+r2*EXI
      c=((x2-x1)/AL)
      sin=((r2-r1)/AL)
      DO 101 K=1,2
      EXI=0.5+GAUSS(K,1)/2.
      r=r1*(1.-EXI)+r2*EXI
      CALL Bmatrix(c,sin,AL,EXI,r,BB)
c     Transpose matrix BB
      DO 60 I=1,6
      DO 60 J=1,4
60    BT(I,J)=BB(J,I)
      CALL MULTIPLY(4,4,6,DE,BB,TEMP)
      CALL MULTIPLY(6,4,6,BT,TEMP,SS)
      DO 70 I=1,6
      DO 70 J=1,6
70    ES(I,J)=ES(I,J)+Pi*AL*r*SS(I,J)*GAUSS(K,2)
101   CONTINUE
      RETURN
      END

      SUBROUTINE TRANSFORMES(c,sin,ES,EST)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
c     Transform element stiffness matrix from local to global coordinates.
      COMMON/STDAT1/NJ,NE,NSJ,NRAR
      DIMENSION ES(6,6),EST(6,6),T(6,6),TT(6,6),PRODUCT(6,6)
      DO 30 I=1,6
      DO 30 J=1,6
      EST(I,J)=0.
30    T(I,J)=0.
      T(1,1)=c
      T(2,1)=-sin
      T(1,2)=sin
      T(2,2)=c
      T(3,3)=1
      T(4,4)=c
      T(5,4)=-sin
      T(4,5)=sin
      T(5,5)=c
      T(6,6)=1
      DO 40 I=1,6
      DO 40 J=1,6
40    TT(I,J)=T(J,I)
      CALL MULTIPLY(6,6,6,ES,T,PRODUCT)
      CALL MULTIPLY(6,6,6,TT,PRODUCT,EST)
      RETURN
      END

      SUBROUTINE ELASTICITY(t,DE,E,Pr)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
c     Generation of elasticty matrix DE
      COMMON/STDAT1/NJ,NE,NSJ,NRAR
      DIMENSION DE(4,4)
      DO 10 I=1,4
      DO 10 J=1,4
10    DE(I,J)=0
      DE(1,1)=E*t/(1.-PR*PR)
      DE(2,1)=DE(1,1)*PR
      DE(2,2)=DE(1,1)
      DE(3,3)=DE(1,1)*t*t/12.
      DE(4,3)=DE(3,3)*PR
      DE(4,4)=DE(3,3)
      DO 20 I=1,3
      DO 20 J=I+1,4
20    DE(I,J)=DE(J,I)
      RETURN
      END

      SUBROUTINE TEMPFORCE(r1,x1,r2,x2,t,Ti,To,ALPHA,FRT,GAUSS,E,Pr)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
c     Element nodal forces in global direction to restrain thermal expansion.
      COMMON/STDAT1/NJ,NE,NSJ,NRAR
      DIMENSION SIGMAR(4),GAUSS(2,2),BB(4,6),BT(6,4),FR(6),PRODUCT(6)
      DIMENSION DE(4,4),FRT(6)
c     Matrix TEMP is for storage of product of matrices
      Pi=3.141592654
      AL=DSQRT((r2-r1)**2+(x2-x1)**2)
      c=((x2-x1)/AL)
      sIN=((r2-r1)/AL)
      CALL ELASTICITY(t,DE,E,Pr)

      CALL SIGMAREST(t,DE,ALPHA,Ti,To,SIGMAR)

c      Write(6,101)gauss(1,1),gauss(1,2),gauss(2,1),gauss(2,2)
c101   format(/,'Gauss:',E13.5,1X,E13.5,/,'Gauss:',E13.5,1X,E13.5)
c      write(6,102)(SIGMAR(II),II=1,4)
c102   format(/,'Sigmar:',4(E13.5,1X))
c      do 104 II=1,4
c      write(6,103)(DE(II,JJ),JJ=1,4)
c103   format(/,'DE',4(E13.5,1X))
c104   continue


      DO 10 I=1,6
10    FR(I)=0.
      DO 20 K=1,2
      EXI=0.5+GAUSS(K,1)/2.
      r=r1*(1-EXI)+r2*EXI
      CALL Bmatrix(c,sIN,AL,EXI,r,BB)
      
c      do 106 II=1,4
c      write(6,105)(BB(II,JJ), JJ=1,6)
c105   format(/,'BB',6(E11.4,1X))
c106   continue

      DO 14 I=1,6
      DO 14 J=1,4
14    BT(I,J)=BB(J,I)


      CALL MULTIPLY(6,4,1,BT,SIGMAR,PRODUCT)

c      write(6,107) (Product(JJ),JJ=1,6)
c107   format(/,'Product',6(E11.4,',')  )


      DO 15,I=1,6
15    FR(I)=FR(I)+(2.*Pi*r)* 0.5*AL*PRODUCT(I)*GAUSS(K,2)
20    CONTINUE

c      write(6,108) (FR(JJ),JJ=1,6)
c108   format(/,'FR',6(E11.4,',')  )




      FRT(1)=FR(1)*c-FR(2)*SIN
      FRT(2)=FR(1)*SIN+FR(2)*c
      FRT(3)=FR(3)
      FRT(4)=FR(4)*c-FR(5)*SIN
      FRT(5)=FR(4)*SIN+FR(5)*c
      FRT(6)=FR(6)

c      write(6,109) (FRT(JJ),JJ=1,6)
c109   format(/,'FRT',6(E11.4,',')  )



      RETURN
      END

      SUBROUTINE SIGMAREST(t,DE,ALPHA,Ti,To,SIGMAR)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
c     Generalized stresses due to temperature rise, with expansion prevented.
      COMMON/STDAT1/NJ,NE,NSJ,NRAR
      DIMENSION DE(4,4),FREE(4),SIGMAR(4)
      FREE(1)=-ALPHA*(Ti+To)/2.
      FREE(2)=Free(1)
      FREE(3)=-ALPHA*(To-Ti)/t
      FREE(4)=FREE(3)
      CALL MULTIPLY(4,4,1,DE,FREE,SIGMAR)
      RETURN
      END

      SUBROUTINE GAUSS7(GAUSS)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
c     Generate 7x2 matrix of constants for Gauss numerical integration.
      COMMON/STDAT1/NJ,NE,NSJ,NRAR
      DIMENSION GAUSS(7,2)
      GAUSS(1,1)=-0.949107912342759
      GAUSS(1,2)=		    0.129484966168870
      GAUSS(2,1)=-0.741531185599394
      GAUSS(2,2)=		    0.279705391489277
      GAUSS(3,1)=-0.405845151377397
      GAUSS(3,2)=		    0.381830050505119
      GAUSS(4,1)=0.
      GAUSS(4,2)=		    0.417959183673469
      GAUSS(7,1)=+0.949107912342759
      GAUSS(7,2)=		    0.129484966168870
      GAUSS(6,1)=+0.741531185599394
      GAUSS(6,2)=		    0.279705391489277
      GAUSS(5,1)=+0.405845151377397
      GAUSS(5,2)=		    0.381830050505119
      RETURN
      END

      SUBROUTINE GAUSS2(GAUSS)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
c     Generate 2x2 matrix of constants for Gauss numerical integration.
      COMMON/STDAT1/NJ,NE,NSJ,NRAR
      DIMENSION GAUSS(2,2)
      const=3.
      GAUSS(1,1)=-DSQRT(const)/3.
      GAUSS(1,2)=		    1.
      GAUSS(2,1)=DSQRT(const)/3.
      GAUSS(2,2)=		    1.
      RETURN
      END

      SUBROUTINE Bmatrix(c,sin,AL,EXI,r,BB)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
c     Generation of the B-matrix, relating generalizeed strains to nodal
c     displacements.
      COMMON/STDAT1/NJ,NE,NSJ,NRAR
      DIMENSION BB(4,6)
      DO 40 I=1,4
      DO 40 J=1,6
40    BB(I,J)=0.
      BB(1,1)=-1./AL
      BB(2,1)=(1.-EXI)*sin/r
      BB(2,2)=(1.-3.*EXI*EXI+2.*EXI*EXI*EXI)*(c/r)
      BB(3,2)=-(-6.+12.*EXI)/(AL*AL)
      BB(4,2)=(6.*EXI-6.*EXI*EXI)*sin/(r*AL)
      BB(2,3)=(AL*EXI)*((EXI-1.)**2)*(c/r)
      BB(3,3)=-(-4.+6.*EXI)/AL
      BB(4,3)=(-1.+4.*EXI-3.*EXI*EXI)*(sin/r)
      BB(1,4)=1./AL
      BB(2,4)=EXI*sin/R
      BB(2,5)=(3.*EXI*EXI-2.*EXI*EXI*EXI)*(c/r)
      BB(3,5)=-(6.-12.*EXI)/(AL*AL)
      BB(4,5)=(-6.*EXI+6.*EXI*EXI)*(sin/(r*AL))
      BB(2,6)=AL*(-EXI*EXI+EXI**3)*(c/r)
      BB(3,6)=-(-2.+6.*EXI)/AL
      BB(4,6)=-(-2.*EXI+3.*EXI*EXI)*(sin/r)
      RETURN
      END

      SUBROUTINE MULTIPLY(N,M,K,A,B,C)
c     Multiply matrices A*B=C. Sizes: A  NxM and  B   MxK
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(N,M),B(M,K),C(N,K)
      DO 11 I=1,N
      DO 11 J=1,K
11    C(I,J)=0.
      DO 10 I=1,N
      DO 10 J=1,K
      SUM=0.
      DO 20 L=1,M
20    SUM=SUM+A(I,L)*B(L,J)
10    C(I,J)=SUM
      RETURN
      END

      SUBROUTINE PLOTDATA
     +(PLNS,PLNPHI,PLMS,PLMPHI,PLOT1,PLOT2,NE,IOUTPUT)
c     Preparation of r-x plot of results. Positive x is upwards; positive
c     r is pointing horizontal to the right.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION PLNS(NE,4),PLNPHI(NE,4),PLMS(NE,4),PLMPHI(NE,4)
      DIMENSION PLOT1(NE,4),PLOT2(NE,4)
c     Determine a suitable factor to magnify(or reduce) stress resultants.
c     Let the multiplier be=FACTNS,or FACTMS, and so on.
      RMAX=-1.e-12
      RMIN=1.e12
      XMAX=-1.e-12
      XMIN=1.e12
      ANSMAX=-1e-12
      ANPHIMAX=-1.e-12
      AMSMAX=-1e-12
      AMPHIMAX=-1.e-12
      DO 10 I=1,NE
      IF(PLOT1(I,1).GT.RMAX)RMAX=PLOT1(I,1)
      IF(PLOT1(I,1).LT.RMIN)RMIN=PLOT1(I,1)
      IF(PLOT1(I,2).GT.XMAX)XMAX=PLOT1(I,2)
      IF(PLOT1(I,2).LT.XMIN)XMIN=PLOT1(I,2)
      IF(ABS(PLOT2(I,1)).GT.ANSMAX)ANSMAX=ABS(PLOT2(I,1))
      IF(ABS(PLOT2(I,2)).GT.ANPHIMAX)ANPHIMAX=ABS(PLOT2(I,2))
      IF(ABS(PLOT2(I,3)).GT.AMSMAX)AMSMAX=ABS(PLOT2(I,3))
      IF(ABS(PLOT2(I,4)).GT.AMPHIMAX)AMPHIMAX=ABS(PLOT2(I,4))
10    CONTINUE
      CONST=5.
      AR=ABS(RMAX-RMIN)
      AX=ABS(XMAX-XMIN)
      AL=AX
      IF(AR.GT.AX)AL=AR

      IF(IOUTPUT.EQ.1)
     * write(6,76)rmax,rmin,xmax,xmin,al
76    format(/,3X,'-------------------------------',
     +'--------------------------------------------------------',/,3X,/
     +,3X,'The above results are sufficient for plotting.',/,3X,
     +'Altenatively, the following may be used for plotting. The ',/,
     +3X,'choice is up to the user.',/,/,3X,
     +'Maximum and minimum coordinates of nodes:',/,3X,
     +'rmax=',e12.5,' rmin=',e12.5,' xmax=',e12.5,' xmin=',e12.5,
     +/,3X,'Maximum structure dimension, Lmax=',e12.5)

      IF(IOUTPUT.EQ.1)
     * write(6,766)ansmax,anphimax,amsmax,amphimax
766   format(/,3X,'Largest absolute values of stress resultants:',/,3x,
     +'Nsmax=',e12.5,' Nphimax=',e12.5,
     +' MSmax=',e12.5,' Mphimax=',e12.5)

      IF(ANSMAX.NE.0.)FNS=AL/(CONST*ANSMAX)
      IF(ANSMAX.EQ.0.)FNS=0.
      IF(ANPHIMAX.NE.0.)FNPHI=AL/(CONST*ANPHIMAX)
      IF(ANPHIMAX.EQ.0.)FNPHI1=0.
      IF(AMSMAX.NE.0.)FMS=AL/(CONST*AMSMAX)
      IF(AMSMAX.EQ.0.)FMS=0.
      IF(AMPHIMAX.NE.0.)FMPHI=AL/(CONST*AMPHIMAX)
      IF(AMPHIMAX.EQ.0.)FMPHI=0.

      IF(IOUTPUT.EQ.1)
     * write(6,77)FNS,FNPHI,FMS,FMPHI
77    format(/,3X,
     +'Multipliers to stress resultants to make their largest',/,3X,
     +'absolute values=Lmax/5:',/,3X,
     +'Ns=',e12.5,' Nphi=',e12.5,' Ms=',e12.5,' Mphi=',e12.5)

c      do 776 ii=1,ne
c      write( 6,778)
c     * PLOT1(ii,1),PLOT1(ii,2),PLOT1(ii,3),PLOT1(ii,4)
c778   FORMAT('PL1',1X,E12.5,1X,E12.5,3X,E12.5,3X,E12.5)
c776   continue

c      do 777 ii=1,ne
c      write( 6,779)
c     * PLOT2(ii,1),PLOT2(ii,2),PLOT2(ii,3),PLOT2(ii,4)
c779   FORMAT('PL2',1X,E12.5,1X,E12.5,3X,E12.5,3X,E12.5)
c777   continue

      DO 50 JM=1,NE

      PLNS(JM,1)=PLOT1(JM,1)
      PLNS(JM,2)=PLOT1(JM,2)

      PLNPHI(JM,1)=PLOT1(JM,1)
      PLNPHI(JM,2)=PLOT1(JM,2)

      PLMS(JM,1)=PLOT1(JM,1)
      PLMS(JM,2)=PLOT1(JM,2)

      PLMPHI(JM,1)=PLOT1(JM,1)
      PLMPHI(JM,2)=PLOT1(JM,2)

      PLNS(JM,3)  =PLOT1(JM,1)+PLOT2(JM,1)*PLOT1(JM,3)*FNS
      PLNS(JM,4)  =PLOT1(JM,2)-PLOT2(JM,1)*PLOT1(JM,4)*FNS

      PLNPHI(JM,3)=PLOT1(JM,1)+PLOT2(JM,2)*PLOT1(JM,3)*FNPHI
      PLNPHI(JM,4)=PLOT1(JM,2)-PLOT2(JM,2)*PLOT1(JM,4)*FNPHI

      PLMS(JM,3)  =PLOT1(JM,1)+PLOT2(JM,3)*PLOT1(JM,3)*FMS
      PLMS(JM,4)  =PLOT1(JM,2)-PLOT2(JM,3)*PLOT1(JM,4)*FMS

      PLMPHI(JM,3)=PLOT1(JM,1)+PLOT2(JM,4)*PLOT1(JM,3)*FMPHI
      PLMPHI(JM,4)=PLOT1(JM,2)-PLOT2(JM,4)*PLOT1(JM,4)*FMPHI

50    CONTINUE
      RETURN
      END


c		 Solution of Banded Symmetrical Equations
c		      For Use in Structural Analysis
c		   Based on Ghali,A. and Neville,A. M.,
c      Structural Analysis :A Unified Classical and Matrix Approach
c       Chapman and Hall,London and New York,sixth ed.,2012,Chapter 24
c----------------------------------------------------------------------
c		     Notation and dimensions of arrays
c							     rows  cols.
c    NB     band width ; see Sec. 24-3 of Ghali and Nevile.
c    NLC    number of load cases
c    [D]    nodal displacements 			     NDF    NLC
c    [F]    nodal restraining forces			     NDF    NLC
c    [S]    stiffness matrix				     NDF    NB
c------------------------------------------------------------------------------
      SUBROUTINE EQSOLN(S,F,D,NDF,NB,NLC)
c     Solution of equilibrium equations [S][D]=-[F], using Cholesky's
c     method; see Sec. 25-11 of Ghali and Neville.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION S(NDF,NB),F(NDF,NLC),D(NDF,NLC)
      IF(S(1,1).LE. 0.0)GO TO 98
      DO 50 J=2,NDF
      JJ=J-1
      JJJ=J-NB+1
      IF(JJJ.LT.1)JJJ=1
      IF(JJ.EQ.1)GO TO 30
      DO 20 I=2,JJ
      II=I-1
      IF(II.LT.JJJ)GO TO 20
      SUM=S(I,J-I+1)
      DO 10 K=JJJ,II
10    SUM=SUM-S(K,I-K+1)*S(K,J-K+1)
      S(I,J-I+1)=SUM
20    CONTINUE
30    SUM=S(J,1)
      DO 40 K=JJJ,JJ
      T=S(K,J-K+1)/S(K,1)
      SUM=SUM-T*S(K,J-K+1)
40    S(K,J-K+1)=T

c     IF(SUM.LE.0.0)GO TO  98
c      if(sum.lt.0.0)write(6,101)j
      if(sum.eq.0.0)go to 98

50    S(J,1)=SUM
c     At this stage,the first column of the band matrix [S] is replaced
c     by e11,e22,... and the remaining elements of the same matrix are
c     replaced by  elements uij using Eq. 24-49 to 24-53 of Ghali and
c     Neville. Now by forward substitution (Eq. 24-48), determine [g]
c     and store in [D].

      DO 95 L=1,NLC
      DO 70 I=1,NDF
      J=I-NB+1
      IF(I.LE.NB)J=1
      SUM=-F(I,L)
      KK=I-1
      IF(J.GT.KK)GO TO 65
      DO 60 K=J,KK
60    SUM=SUM-S(K,I-K+1)*D(K,L)
65    D(I,L)=SUM
70    CONTINUE
      DO 80 I=1,NDF
80    D(I,L)=D(I,L)/S(I,1)

c     By Eq. 24-46 of Ghali and Neville and backward substitution(Eq.24-
c     47),determine the required displacement values and store in [D].
      DO 95 II=1,NDF
      I=NDF-II+1
      J=I+NB-1
      IF(J.GT.NDF) J=NDF
      SUM=D(I,L)
      KK=I+1
      IF(KK.GT.J) GO TO 90
      DO 85 K=KK,J
85    SUM=SUM-S(I,K-I+1)*D(K,L)
90    D(I,L)=SUM
95    CONTINUE
      GO TO 99
98    CONTINUE
      WRITE(6,102)J
102   FORMAT(/,3X,
     +'Stiffness matrix is not positive definite. Zero value encountered
     +',/,3X,
     +'on the diagonal in solution of equations on row:',I4)
101   format(/,3x,
     +'Stiffness matrix is not positive definite. Negative value encount
     +ered',/,3X,
     +'on the diagonal in solution of equations on row:',I4)

      STOP
99    CONTINUE
      RETURN
      END
